# Copyright (C) 2010 Julien VAUBOURG
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

use strict;
package Capture;
use PSM::Zones;


# class Capture
#
# UTILISATION
#
# Capture se charge de generer l'arborescence complete des bases RRD depuis les informations contenues dans des fichiers
# de type pcap produits par tcpdump lors de la capture de la sonde.
#
# 1) Transformation des binaires
# Les fichiers pcap sont lus avec tcpdump, qui ne ressortira que les entetes des paquets. Ces informations sont redirigees
# vers des fichiers textes, qui contiendront les informations d'un paquet par ligne.
#
# 2) Alimentation des RRD
# Les fichiers textes sont ensuite scannes dans l'ordre (cet ordre doit imperativement correspondre a l'ordre de creation des fichiers,
# d'ou la modification du nom des fichiers textes), ligne par ligne. Pour chaque ligne on identifie la date, l'IP source, de destination, la taille du paquet,
# les ports utilises et le protocole. Chaque port et chaque protocole pour chaque couple source-destination debouche sur la creation d'un RRD,
# et eventuellement des dossiers de son chemin. Ce chemin se cree toujours de la facon : RRDIR/ipSource/ipDestination/ressource.rrd
# <ressource.rrd> correspond donc aux transferts dans le temps de ipSource vers ipDestination en utilisant cette ressource (un port ou un protocole particulier).
# Un systeme de buffer permet de reduire considerablement le nombre d'ecriture sur le disque en limitant le nombre de mises a jour des RRD (les donnees s'additionnent
# sur nbSecondes pour creer une valeur, et ces valeurs sont mises a jour par paquets de nbParametres - cf. ajouter() pour plus de details).
#
# 3) Vidange des updates
# Le systeme de buffer impose de mettre a jour une derniere fois l'ensemble des bases RRD avec les valeurs restantes.
# Lors de cette derniere update, on renomme les bases en incluant la taille totale des donnees cumulees pour le flux qu'elles representent.
# Les fichiers prennent alors la forme : ressource.total.rrd (en bits !)
# Recuperer et additionner les donnees pour obtenir le total de chaque RRD est long et imprecis (RRDTool adapte les valeurs pour ses graphiques),
# cette solution corrige le probleme.


###
### VARIABLES DE CLASSE
###

our $nbSecondes = 60; # Nombre de secondes entre le premier et le dernier paquet (date de capture) pour creer une valeur dans les RRD (cf. ajouter())
our $nbParametres = 30; # Nombre de valeurs a chaque mise a jour d'un RRD (cf. ajouter())


###
### CONSTRUCTEUR
###

# @arg Op. : Dossier des dumps binaires de tcpdump
sub new {
	my $classe = shift;
	my $this = {};

	# Instanciation
	bless ($this, $classe);

	# Retour de la reference
	return $this;
}


###
### METHODES DE CLASSE
###

# Transforme les fichiers pcap du dossier de l'instance en fichiers textes qui ne contiennent que les entetes des paquets
# (un par ligne). Ces fichiers sont stockes dans un repertoire text lui-meme contenu dans le dossier de l'instance
# et eventuellement cree pour l'occasion.
sub bin2txt {
	my $this = shift;

	# Tableau des differents timestamp differents trouves dans les noms de fichiers.
	# Un timestamp different est defini a chaque arret de tcpdump lors de la capture de 
	# la sonde. Le nombre de timestamps trouves moins un correspondra donc au nombre de coupure
	# lors de la capture.
	my @coupures; 

	if(-d "$RRD::root/dumps/text") {
		print "ERREUR: $RRD::root/dumps/text existe deja, vous devez le supprimer avant de recommencer une transformation des dumps\n";
		return 0;
	}

	# Creation du dossier text qui contiendra les equivalents textes des binaires
	`mkdir $RRD::root/dumps/text`;
	print "Binaires > txt...\n";

	# Ouverture du dossier contenant les fichiers binaires pcap
	opendir(CAPDIR, "$RRD::root/dumps") or die "ERREUR: Impossible d'ouvrir $RRD::root/dumps\n";

	# Pour chaque fichier du dossier
	foreach my $fichier (readdir CAPDIR) {

		# On ne veut pas de . et .. ni text ou autres fichiers/dossiers qui ne sont pas des dumps
		if($fichier =~ /^dump[0-9]+_[0-9]*$/) {

			# Isolation du timestamp et du numero du nom du fichier. TIMESTAMP_NB : timestamp est defini au lancement de tcpdump
			# et nb correspond a l'ordre des fichiers pour chaque timestamp. Tcpdump demarre un nouveau fichier a chaque fois que
			# le precedent a atteint la taille maximale, et les numerote a partir de 1, le premier n'ayant aucun numero. Cette numerotation
			# recommence a chaque nouveau timestamp (donc a chaque coupure de tcpdump).
			my ($timestamp, $numero) = $fichier =~ /^dump([0-9]+)_([0-9]*)$/;

			# Ajout du timestamp dans le tableau des coupures, si il n'a pas deja ete ajoute
			push(@coupures, $timestamp) if !grep($_ eq $timestamp, @coupures);

			# Les fichiers du dossier etant lus dans l'ordre lexicographique, des zeros sont ajoutes devant les nombres
			$numero = sprintf('%04d', $numero);
			$timestamp = sprintf('%014d', $timestamp);

			# Lecture du fichier et redirection des donnees dans le fichier texte
			# -v : Uniquement les entetes des paquets
			# -N : Ne jamais convertir des fragments d'IP en domaines
			# -S : Valeurs absolues
			# -n : Ne jamais convertir les IP en nom de domaine (qui ne seront donc visibles que dans les requetes DNS)
			# -tt : Dates en timestamps UNIX
			# -r : Capturer a partir du fichier de capture indique
			# tcp or udp : Filtrage des paquets par protocoles
			`tcpdump -vNSn -tt -r $this->{dossier}/$fichier > $this->{dossier}/text/${timestamp}_$numero tcp or udp`;
		}
	}

	close CAPDIR;
	print "OK : Il y a eu ".(@coupures-1)." interruption(s) lors de la capture des donnees.\n"
}

# Fonction essentielle de PSM::Capture, lire() est chargee de lire les fichiers textes generes par bin2txt() et de construire l'arborescence des RRD.
# Ces fichiers sont lus un par un dans l'ordre chronologique et ligne par ligne (une ligne = un paquet).
# Pour chaque paquet on construit ses RRD dans les dossiers : ipSourceDuPaquet/ipDestinationDuPaquet/
# Deux RRD sont renseignes pour chaque paquet : PORTx.rrd et PROTO_x.rrd
# Le port choisis (entre le port sortant du paquet et le port entrant) est le port le plus petit. Et si ce port depasse 1024, on considere que c'est
# un port dynamique, on considere donc qu'il s'agit du port Autres. A noter que c'est imprecis : enormement d'applications utilisent des ports depassant
# 1024, mais elles ne sont pas tres courantes, et que c'est le seul moyen simple d'eliminer les ports dynamiques qui n'ont aucune signifaction.

sub lire {
	my $this = shift;

	# Si le dossier text contenant les fichiers des entetes des paquets n'existe pas encore, on
	# les fait se generer
	bin2txt($this) if !-d "$RRD::root/dumps/text";

	# Ouverture du dossier des fichiers textes
	opendir(CAPDIR, "$RRD::root/dumps/text") or die "ERREUR: Impossible d'ouvrir $RRD::root/dumps/text\n";
	print "\nBEGIN (This may take some time ;)\n";

	# Initialisation des compteurs qui ne servent pas a grand chose mais qu'on peut faire facilement alors pourquoi s'en priver
	$this->{create} = 0;
	$this->{update} = 0;
	$this->{paquets} = 0;
	$this->{paquetsUtilises} = 0;

	# Pour chaque fichier texte d'entetes de paquets, parcourus dans l'ordre chronologique (lexicographique)
	foreach my $fichier (sort readdir CAPDIR) {

		# On ne veut pas de . et ..
		if(!($fichier =~ /^\./)) {

			# Ouverture du fichier en cours
			open(CAPTURE, "$RRD::root/dumps/text/$fichier") or die "ERREUR: Impossible d'ouvrir $RRD::root/dumps/text/$fichier\n";

			# Pour chaque entetes de paquet du fichier
			while (<CAPTURE>) {

				# Extraction des donnees lues dans les entetes du paquet
				my ($temps, $proto, $valeur, $ipSrc, $portSrc, $ipDst, $portDst) = $_ =~ /^([0-9]+)\.[0-9]+ .*IP \(.* proto:? ([A-Z]+) .* length:? ([0-9]+)\) ([0-9a-f:.]+)\.([0-9]+) > ([0-9a-f:.]+)\.([0-9]+):/;

				# Si l'extraction semble etre logique et si ca n'est pas un paquet ICMP (inutile, et relou a traiter)
				if($portDst > 0 && $proto =~ /(TCP|UDP)/) {

					# Le port qu'on gardera sera le plus petit
					my $port = $portSrc < $portDst ? $portSrc : $portDst;

					# Si ce port depasse 1024 on le range dans Autres (probablement un port dynamique)
					$port = 'Autres' if $port > 1024 && $port != 8080;

					# Ajout de la taille du paquet dans le RRD du flux de cette ip source a cette ip destination,
					# pour le port et le protocole concerne
					ajouter($this, "$ipSrc/$ipDst/PROTO_$proto.rrd", $temps, $valeur);
					ajouter($this, "$ipSrc/$ipDst/PORT$port.rrd", $temps, $valeur);

					# Une liste de ports et de protocoles et construite, elle servira pour l'etablissement des rapports (groupes de possibilites)
					push(@{$this->{ports}}, $port) if !grep($_ eq $port, @{$this->{ports}});
					push(@{$this->{protos}}, $proto) if !grep($_ eq $proto, @{$this->{protos}});

					# Le compteur d'une des variables qui sert pas a grand chose mais qu'on met quand meme
					$this->{paquetsUtilises}++;
				}

				# Une autre de ces variables
				$this->{paquets}++;
			}
		
			close CAPTURE;
		}
	}

	close CAPDIR;
	print "Vidange...\n";

	# Une fois que tous les paquets de tous les fichiers ont ete pris en compte, on reparcours une derniere fois l'ensemble des RRD generes pour vider les valeurs
	# qui restent dans leur entree du buffer. On en profite pour inclure leur total dans le nom de leur fichier (ressource.total.rrd).
	while(my ($rrd, $buffer) = each %{$this->{buffer}}) {

		# Ajout de la valeur en cours - qui attendait de depasser nbSecondes - dans la liste des valeurs
		$buffer->{valeurs} .= "$buffer->{temps}:$buffer->{total}";

		# Envoi de la liste des valeurs dans le RRD
		`rrdtool update $RRD::root/rrd/$rrd $buffer->{valeurs}`;

		# Incrustation du total transmis lors du flux, dans le nom du RRD (ressource.total.rrd)
		$rrd = substr($rrd, 0, rindex($rrd, '.'));
		`mv $RRD::root/rrd/$rrd.rrd $RRD::root/rrd/$rrd.$buffer->{total}.rrd`;

		# Liberation de la memoire occupee par ce RRD dans la hash map (ne sert surement pas a grand chose)
		delete $this->{buffer}->{"$rrd.rrd"};

		# Et hop : une autre variable qui sert pas grand chose qui s'incremente
		$this->{update}++;
	}

	# Sauvegarde de la liste de ports dans un fichier .list (considere comme un Groupe de possibilites, par la suite)
	open(PORTS, ">$RRD::root/rrd/PORT.list");
	print PORTS join("\n", @{$this->{ports}});
	close PORTS;

	# Sauvegarde de la liste des protocoles
	open(PROTOS, ">$RRD::root/rrd/PROTO_.list");
	print PROTOS join("\n", @{$this->{protos}});
	close PROTOS;

	# Et c'est ici qu'on retrouve toutes ces variables qui ne servent pas a grand chose, mais qui n'avaient que pour but d'afficher un rapport... qui ne sert pas a grand chose :o
	print "\n----------\n";
	print "\nTHE END - Enjoy!\n";
	print "Paquets: $this->{paquets}\n";
	print "Paquets utilises: $this->{paquetsUtilises}\n";
	print "Creates: $this->{create}\n";
	print "Updates: $this->{update}\n";
	print "Ports: ".@{$this->{ports}}."\n";
	print "Protos: ".@{$this->{protos}}."\n";
	print "----------\n\n";
}

# Ajoute une valeur a un RRD et le cree si il n'existe pas, ainsi que l'arborescence de dossiers dans lequel il doit se trouver.
# Pour cela, un buffer est utilise.
#
# Le systeme de buffer :
# Pour chaque RRD (defini par un chemin de type source/destination/ressource.rrd), on cree une entree dans une hash map qui contiendra differentes informations.
# Afin de limiter les ecritures sur le disque on limite les update des RRD en rassemblant les valeurs de deux faons :
# - Une information <total> permet de cumuler la taille de tous les paquets rencontres pour ce type de flux, depuis le debut de la capture. 
#   Une information <temps> est initialisee a la creation du RRD avec le timestamp du premier paquet rencontre. A chaque fois qu'un nouveau paquet est rencontre,
#   on se contente d'ajouter sa longueur au <total> du RRD. Si le timestamp du paquet est superieur de nbSecondes a l'information <temps>, on ajoute le couple temps:total
#   a la suite de <valeurs> et on incremente <nbValeurs>. A titre d'exemple, au lieu de dire au RRD que 2ko ont ete transferes toutes les 10 secondes pendant 1 minute,
#   on lui dira ainsi que 120ko ont ete transferes au bout d'une minute. Les graphiques de temps etant lus sur environ une semaine, cette perte de precision ne derange
#   en rien, et permet de limiter considerable les update des RRD (il y souvent plusieurs dizaines de paquets pour une seule seconde).
# - Les couples timestamp:total stockes dans <valeurs> sont envoyes au RRD a chaque fois que <nbValeurs> atteint nbParametres, RRDTool permettant de renseigner plusieurs
#   valeurs a la fois lors d'un update. Ceci reduit encore le nombre d'acces au disque.
#
# @arg : rrd - Nom du fichier RRD a creer ou completer
# @arg : temps - Timestamp de la date du paquet pour lequel on souhaite ajouter la taille
# @arg : valeur - Taille du paquet a ajouter dans le RRD (le length, en bits)

sub ajouter {
	my ($this, $rrd, $temps, $valeur) = @_;

	# Le RRD a deja ete cree, au moins une valeur est deja dans on buffer
	if(defined $this->{buffer}->{$rrd}) {

		# On ajoute la valeur a ajouter au total du TTD (les RRD prennent toujours le total en update : chaque update doit proposer un nombre toujours plus grand)
		$this->{buffer}->{$rrd}->{total} += $valeur;

		# La derniere valeur a plus de <nbSecondes> d'interval (en terme de date du paquet) avec le dernier ajout dans valeurs :
		# on ajoute ce couple timestamp:valeur au stock de valeurs
		if($temps-$Capture::nbSecondes > $this->{buffer}->{$rrd}->{temps}) {

			# Y'a-t-il suffisament de valeurs en stocks pour envoyer, avant d'ajouter cette valeur ?
			if($this->{buffer}->{$rrd}->{nbValeurs} >= $Capture::nbParametres) {

				# Mise a jour des <nbParametres> valeurs dans le RRD
				`rrdtool update $RRD::root/rrd/$rrd $this->{buffer}->{$rrd}->{valeurs}`;
				$this->{update}++;

				# Reinitialisation du stock de valeurs en attendre d'etre validees
				$this->{buffer}->{$rrd}->{valeurs} = '';
				$this->{buffer}->{$rrd}->{nbValeurs} = 0;
			}

			# Ajout de la valeur au stock
			$this->{buffer}->{$rrd}->{valeurs} .= "$this->{buffer}->{$rrd}->{temps}:$this->{buffer}->{$rrd}->{total} ";

			# Une valeur en plus dans le stock
			$this->{buffer}->{$rrd}->{nbValeurs}++;

			# Mise a jour du <temps> : c'est reparti pour une accumulation de paquets jusqu'au premier qui aura plus de <nbSecondes> secondes
			# de difference avec ce temps
			$this->{buffer}->{$rrd}->{temps} = $temps;
		}

	# Le flux concerne par ce RRD n'a jamais ete rencontre : on cree le RRD et on met la premier valeur en cache
	} else {

		# Arborescence de dossiers du RRD (sans le RRD lui-meme)
		my $dossier = substr("$RRD::root/rrd/$rrd", 0, rindex("$RRD::root/rrd/$rrd", '/'));

		# Construction de cette arborescence (RRDIR/ipSource/ipDestination) si le dossier final (au minimum) n'existe pas
		`mkdir -p $dossier` if !-d $dossier;

		# Creation de la base RRD elle-meme
		my $resolutionX2 = $RRD::resolution * 2;

		# La syntaxe de creation du fichier RRD est la suivante :
		#
		# rrdtool create <Nom du fichier> \
		# --step <Duree entre deux valeurs> \
		# DS:<nom_variable>:<type>:<heartbeat>:<min>:<max>
		#	Avec :
		#		* nom_variable est le nom de la variable dans la base de donnees,
		#		* type est le type de la donnee. COUNTER indique qu'il s'agit d'une donnee qui ne fait que s'incrementer au cours du temps,
		#		* heartbeat est l'intervalle maximal d'echantillonnage exprimee en secondes,
		#		* min est la valeur minimale,
		#		* max est la valeur maximale. Non pas du compteur mais de la difference entre les valeurs du compteur entre deux echantillonnages.
		#
		# Viennent ensuite les lignes decrivant l'echantillonnage dans le temps.
		#
		# La syntaxe est la suivante :
		#
		# RRA:<methode_calcul>:<valeurs_perdues>:<nombre_echantillons>:<nombre_valeurs>
		#	Avec:
		#		* methode_calcul indique la maniere de calculer la valeur finalement stocke dans la base de donnees, ici en faisant une moyenne (AVERAGE),
		#		* valeurs_perdues indique la tolerance par rapport aux valeurs fournies. Ici le calcul est realise meme si 50% des valeurs sont manquantes,
		#		* nombre_echantillons indique le nombre d'echantillons sur lesquels est applique la mthode de calcul. Le calcul fournit une valeur pour la base,
		#		* nombre_valeurs donne le nombre de valeurs stockees dans la base.
		
		`rrdtool create $RRD::root/rrd/$rrd --step $nbSecondes --start $RRD::debut DS:cpt:COUNTER:$resolutionX2:U:U RRA:AVERAGE:0.5:1:$RRD::resolution`;
		$this->{create}++;

		# Initialisation de son buffer et ajout de la premiere valeur a l'interieur de celui-ci
		$this->{buffer}->{$rrd} = {
			'temps' => $temps,
			'total' => $valeur,
			'valeurs' => '',
			'nbValeurs' => 0
		};
	}
}

# Generation du fichier config/nomsServeurs.hash permettant de facilement associer une IP Internet avec son nom de serveur
sub nomsServeurs {
	my $this = shift;

	# Toutes les IP internet rencontres dans les dumps
	my $listeIpInternet = Zones::listerIP('internet');
	my $serveurs = "\%CategoriesInternet::nomsServeurs = (\n";

	# Pour chacune de ces IP
	foreach (@$listeIpInternet) {
		my $CNAME = `host $_`;

		# Si un nom de serveur (CNAME) a t trouv
		if($CNAME !~ /not found/) {

			# On isole le nom de serveur et on l'ajoute  la variable du hash de retour
			$CNAME = substr($CNAME, rindex($CNAME, ' ')+1);
			$CNAME = substr($CNAME, 0, rindex($CNAME, '.'));

			my @CNAME = split('.', $CNAME);

			# On ne garde que les deux dernires parties des noms de serveur
			if(@CNAME > 2) {
				$CNAME = pop @CNAME;
				$CNAME = pop(@CNAME).$CNAME;
			}

			# On ajoute l'association au hash final
			$serveurs .= "'$_' => '$CNAME',\n";
		}
	}

	$serveurs .= ');';

	# Ecriture du fichier
	open(SERVEURS, ">$RRD::root/config/nomsServeurs.hash");
	print SERVEURS $serveurs;
	close SERVEURS;

	print "Fichier domaines.hash gnr dans config/.\n";
}

1; # Obligatoire
