# Copyright (C) 2010 Julien VAUBOURG
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

use strict;
use PSM::GroupeRRD;
package Flux;


# class Flux
#
# UTILISATION
#
# Cette classe a pour but de rassembler les donnees en considerant qu'on observe des flux : elle tentera donc toujours de trouver les
# information de A vers B et de B vers A, en les rassemblant dans des tableaux communs (si on demande a observer les donnees pour PROTO_
# par exemple, le tableau LaTeX qui en ressortira proposera une colonne avec les labels de tous les protocoles rencontres de A vers B comme
# de B vers A, une colonne avec les donnees de A vers B et une autre avec celles de B vers A). Les tableaux sont tries par addition des deux
# direction (A>B + B>A pour chaque ligne). Lorsqu'on demande un graphique, PSM::Flux renverra un tableau des deux graphiques, un pour chaque sens.
# A noter qu'on demandera quasiment toujours des camemberts (les graphiques de temps etant de toutes manieres trop larges pour etre presentes
# cote a cote).
#
# Si on demande a observer une ressource qui n'est pas un groupe de possibilites (comme PROTO_ ou PORT), donc un .rrd avec un joker (*),
# PSM::Flux considerera qu'on souhaite un details par IP. Par defaut, la fonction requete() de Zones defini l'IP source de la ressource comme label
# de toutes les instances de PSM::GroupeRRD qu'elle cree, lorsqu'on lui fourni ce type de ressource. Avec l'option ipEntrants de requete(),
# on obtient la meme chose mais en observant ce que recoivent les IP de la zone destination, de la zone source, en mettant l'IP de la zone
# destination en label (cf. fonction requete de Zones). On peut ainsi ainsi creer facilement des lignes avec les IP d'une zone et les flux dans
# les deux sens pour chacun des IP.


###
### CONSTRUCTEUR
###

# @arg : A - Une zone ou une GZone
# @arg : B - Idem
# @arg : ressource - Que veut-on observer et comparer ? Ce peut etre un groupe de possibilites (PROTO_, PORTS), un fichier rrd avec joker (*) ou un fichier rrd.
#		     Si c'est un fichier avec joker, on considerera qu'on souhaite un details par adresses IP (cf. fonction requete() de PSM::Zones pour le fonctionnement
#		     de la ressource)
# @arg Op. : label - Ce label represente la categorie des donnees observees (ex: "Applications" si on regardes les ports, "IP" si on demande un detail par IP, etc).
# 		     Il s'affichera en vertical pour preciser la nature des labels observes sur chaque ligne.
# @arg Op. : limite - Nombre maximum de lignes dans le tableau final (permet de creer un top X, puisque les lignes sont triees)
sub new {
	my ($classe, $A, $B, $ressource, $label, $limite) = @_;

	# Si la ressource n'est pas un groupe de possibilites, on considere qu'on souhaite un details par IP
	my $detailsIP = index($ressource, '.rrd') != -1;

	# VARIABLES D'INSTANCE
	my $this = {

		# De A vers B
		'A' => new GroupeRRD(Zones::requete($A, $B, $ressource), ($detailsIP ? 'Envoyé à '.Zones::label($B)."||$label ".Zones::label($A) : Zones::label($A).' vers '.Zones::label($B)."||$label")),

		# De B vers A
		# Si c'est un details par IP l'option ipEntrants nous fournira une liste de GroupeRRD avec les labels de A
		'B' => $detailsIP ?
			new GroupeRRD(Zones::requete($B, $A, $ressource, 'ipEntrants'), 'Reçu de '.Zones::label($B)."||$label ".Zones::label($A), 1) :
			new GroupeRRD(Zones::requete($B, $A, $ressource), Zones::label($B).' vers '.Zones::label($A)."||$label", 1),

		'detailsIP' => $detailsIP,
		'zoneA' => $A,
		'zoneB' => $B,
		'limite' => !defined $limite ? 0 : $limite,
		'label' => $label
	};

	# Instanciation
	bless($this, $classe);

	# Aucune source du flux n'existe : avortement de l'instanciation
	if(!$this->{A} && !$this->{B}) {
		print "ATTENTION: La source et la destination du flux n'ont aucune valeur\n" if $RRD::debug;
		return 0;
	}

	# Une des deux directions du flux n'existe pas : on renvoit donc juste le GroupeRRD de la direction qui existe
	return $this->{A} if $this->{A} && !$this->{B};
	return $this->{B} if $this->{B} && !$this->{A};

	# Construction des donnees
	$this->lire();

	# Limitation des des GroupeRRD pour la generation eventuelle de leurs graphiques.
	# Le faire avant la fonction lire() n'aurait pas permis de trier en additionnant les valeurs des GroupeRRD (certaines auraient pu disparaitre
	# lors de la limitation des GroupeRRD).
	$this->{A}->{limite} = $limite;
	$this->{B}->{limite} = $limite;
	$this->{A}->lire();
	$this->{B}->lire();

	# Retour de la reference
	return $this;
}


###
### METHODES D'INSTANCE
###

# Creation des tableaux totaux (totauxA + totauxB) et labels (tries par totaux)
sub lire {
	my $this = shift;

	# On tente d'etablir la liste de tous les labels des GroupesRRD rencontres pour A comme B, sans doublons.
	# On commence par ajouter tous ceux de A
	my @labels = (@{$this->{A}->{labels}});
	my @totaux;

	# Puis on ajoute tous ceux de B qui ne sont pas deja dans la liste
	foreach my $label (@{$this->{B}->{labels}}) {
		push(@labels, $label) if !grep($_ eq $label, @labels);
	}

	# Pour chacun de ces labels
	foreach my $label (@labels) {
		my $total = 0;

		# On additionne selon si A, B, ou les deux, possedent une valeur pour le label courant
		$total += defined $this->{A}->{totaux}->{$label} ? $this->{A}->{totaux}->{$label} : 0;
		$total += defined $this->{B}->{totaux}->{$label} ? $this->{B}->{totaux}->{$label} : 0;

		# Ajout du couple label/total dans le tableau des totaux (une hash map ne permettrait pas de trier par la suite)
		push(@totaux, [$label, $total]);
	}

	# Tri du tableau des totaux selon le total de chaque entree
	@totaux = sort { return @$b[1] <=> @$a[1] } @totaux;

	# Redefinition de la limite, pour ne garder que les X premiers
	$this->{limite} = (!$this->{limite} || $this->{limite} > @totaux ? @totaux : $this->{limite}) - 1;

	# Creation du tableau de labels final, tries selon l'addition des totaux, c'est lui qu'on lira et qui ne contient donc que le
	# nombre d'entrees desire
	push(@{$this->{labels}}, @{@totaux[$_]}[0]) for 0..$this->{limite};
}

# Export des donnees sous forme d'un tableau LaTeX a trois colonnes (labels, A > B, B > A || labels, Recu de B, Envoye a B) ou d'un
# tableau des deux graphiques cote a cote.
# @return : Code LateX
sub toLaTeX {
	my ($this, $elements) = @_;
	my $retour;

	my $totalA = 0;
	my $totalB = 0;

	# On souhaite un camembert
	if($elements =~ /cam/i) {
		$retour .= "\\begin{center}\n"
			."\\vspace{2mm}\n"
			. "\\begin{tabular}{ >{\\centering}m{8cm} >{\\centering}m{8cm} }\n"
			."\\legende{$this->{A}->{label}} & \\legende{$this->{B}->{label}} \\tabularnewline\n"
			.$this->{A}->{graphique}->toLaTeX($elements)." & ".$this->{B}->{graphique}->toLaTeX($elements)
			."\\end{tabular}\n"
			."\\vspace{6mm}\n"
			."\\end{center}\n";
	}

	# On souhaite un tableau recapitulatif
	if($elements =~ /tab/i) {
		$retour .= "\\begin{center}\n"
			."\\vspace{2mm}\n"
			."\\rowcolors{1}{tab0}{tab1}\n"
			."\\begin{tabular}{c|c >{\\centering}m{6cm} >{\\centering}m{6cm}|}\n"
			."\\hhline{~~--}\n \\rowcolor{tabLabel}\n \\multicolumn{2}{c|}{} & \\label{$this->{A}->{label}} & \\label{$this->{B}->{label}} \\tabularnewline\n \\hhline{~-~~}\n";

		# Pour chaque label (du type de la variable d'instance label, qui s'affichera sur le cote en vertical)
		for(0..@{$this->{labels}}-1) {
			my $label = @{$this->{labels}}[$_];

			# Affichage des valeurs en octets et en unites internationales
			my $totauxA = defined $this->{A}->{totaux}->{$label} ? $this->{A}->{totaux}->{$label} : 0;
			my $totauxB = defined $this->{B}->{totaux}->{$label} ? $this->{B}->{totaux}->{$label} : 0;

			# Affichage des deux valeurs
			$retour .= "\\cellcolor{white} & \\cellcolor{tabLabel} \\label".($this->{detailsIP} ? 'IP' : '')."{$label} & ".RRD::adapterUnite($totauxA)." & ".RRD::adapterUnite($totauxB)." \\tabularnewline\n";

			# Mise à jour des totaux globaux
			$totalA += $totauxA;
			$totalB += $totauxB;
		}

		# Ajout du libelle vertical 
		$retour .= "\\cellcolor{white}";
		$retour .= "\\multirow{-".(@{$this->{labels}}+1)."}*{\\begin{sideways}\\indication{".$this->{label}.($this->{detailsIP} ? ' '.Zones::label($this->{zoneA}) : '')."}\\end{sideways}}";

		# Ajout des totaux globaux
		$retour .= "&\\cellcolor{tabLabel} \\label{Totaux} & \\cellcolor{tabLabel} \\textbf{".RRD::adapterUnite($totalA)."} & \\cellcolor{tabLabel} \\textbf{".RRD::adapterUnite($totalB)."} \\tabularnewline\n";

		$retour .= "\\hhline{~---}\n"
			."\\end{tabular}\n"
			."\\vspace{6mm}\n"
			."\\end{center}\n";
	}

	# Retour du code LaTeX
	return $retour;
}

# Methode invoquee lors de la destruction de l'instance
sub DESTROY {
	my $this = shift;

	undef $this->{labels};
	undef $this->{A};
	undef $this->{B};
}

1; # Obligatoire
