# Copyright (C) 2010 Julien VAUBOURG
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

use strict;
use PSM::RRD;
package GroupeRRD;


# class GroupeRRD
#
# UTILISATION
#
# PSM::GroupeRRD permet de reunir des instances de PSM::RRD pour en faciliter leur traitement et leur mise en commun, voire leur fusion.
# Il dispose d'un tableau source d'instances de PSM::RRD (donne directement ou issu d'une recherche liee a un nom fichier donne avec un joker (*)),
# et propose un total des donnees cumulees par les RRD qu'il reference (addition des champs total des instances PSM::RRD).
#
# Il met egalement a disposition un nouveau graphique qui aura une ou plusieurs courbes, selon la valeur de sa variable d'instance fusion, definie
# lors de son instanciation. Si fusion est vrai, le graphique qui sera trace ne presentera qu'une unique courbe qui representera le cumul des evolutions
# de tous les RRD dans le temps. Ce dernier aura alors pour libelle la valeur du champ label a definir aussi dans l'instanciation. Sinon, le graphique aura autant de courbes
# que l'instance du Groupe a de PSM::RRD dans son tableau source, et elles seront libellees avec leur propre label (le label de la variable d'instance des PSM::RRD).
# PSM::GroupeRRD permet aussi de sortir un tableau en LaTeX des totaux de ses PSM::RRD, par ordre decroissement et d'un nombre eventuellement limite par sa variable d'instance
# limite (ce qui constitue un "top").
#
# Si les PSM:RRD ont tous des labels qui peuvent etre rassembles dans un meme type (comme souvent : Protocoles, Applications, etc), il est
# possible d'indiquer ce type dans la variable label de l'instance du Groupe en utilisant un double pipe (ex: label = "Entre A et B||Protocoles").


###
### CONSTRUCTEUR
###

# @arg : source - Fichier RRD (depuis RRD::root/rrd) definie avec un joker ou la reference d'un tableau d'instances de PSM::RRD
# @arg Op. : label - Libelle associe aux valeurs observees, sort principalement pour le label du graphique associe en cas de fusion (|| pour indiquer  la suite le type de donne pour le tableau LaTeX, optionnel)
# @arg Op. : fusion - Booleen determinant si les courbes doivent fusionner a l'affichage du graphique (si vrai, une seule courbe apparaitra sous le libelle de la variable label, sinon il y aura autant de courbe que de PSM::RRD avec leur propre label)
# @arg Op. : limite - Les PSM::RRD qui seront pris en compte seront les n premiers qui auront le plus gros total. limite defini ce n (permet de creer un top)
sub new {
	my ($classe, $source, $label, $fusion, $limite) = @_;

	# Separation du label et du type de donnees eventuellement indique
	my @label = split(/\|\|/, $label);

	# VARIABLES D'INSTANCE
	my $this = {
		'source' => $source, # Tableaux de PSM::RRD
		'label' => @label[0], # Label du regroupement
		'donnee' => defined @label[1] ? @label[1] : undef, # Type de donnees (ex: Applications, Protocoles, etc)
		'fusion' => defined $fusion ? $fusion : 0, # Les courbes du graphique associe doivent-elles fusionner ?
		'limite' => $limite # On ne garde que les n premiers PSM::RRD de source qui ont le plus gros total (le total du Groupe, lui, les prend tous en compte)
	};

	# Instanciation
	bless($this, $classe);

	# Si le groupe ne contient aucune source, on retourne 0 et on avorte son instanciation
	if(!$this->{source} || !@{$this->{source}}) {
		print "ATTENTION: Le groupe ne contient aucune source existante\n" if $RRD::debug;
		return 0;
	}

	# Construction des donnees (addition des totaux dans total)
	$this->lire();

	# Association d'un graphique (qui prendra en compte le fait que la source est un tableau, la variable fusion et la variable limite)
	$this->{graphique} = new Graphique($this);

	# Si le label est un numero, c'est que c'est un port, on le remplace donc par l'application associee
#	if($this->{label} =~ /^[0-9]+$/) {
#
#		# Recherche de l'application correspondante dans /etc/services
#		my $protocole = uc(`grep '[[:space:]]$this->{label}/' /etc/services | head -n1 | cut -f1`);
#		chomp($protocole);
#
#		# Remplacement du numero de port si une application a ete trouvee
#		$this->{label} = $protocole if $protocole ne '';
#	}

	# Retour de la reference
	return $this;
}


###
### METHODES D'INSTANCE
###

# Trie le tableau source de PSM::RRD en fonction de leur total.
# Calcul le total du Groupe en cumulant TOUS les total des PSM::RRD.
# Cree une variable d'instance labels de type tableau qui contiendra les label des n premiers PSM::RRD de source (limite represente n). Ce tableau est utilise pour conserve l'ordre des sources (une hash map ne conserve aucun ordre).
# Cree une variable d'instance totaux de type hash map qui contiendra le total pour chacun des PSM::RRD en fonction de leur label :
# 	'totaux' => ( 'label0' => valeur0, 'label1' => 'valeur1' )
# Cree enfin une variable d'instance pourcentage qui, de la meme facon que totaux contiendra les pourcentages que represente les PSM::RRD dans le Groupe (les pourcentages sont calcules en fonction du total du groupe qui a ete calcule avec TOUS les PSM::RRD de source).
sub lire {
	my $this = shift;
	$this->{total} = 0;

	# La nombre maximum de PSM::RRD a conserver pour l'etablissement des differentes variables (top n, puisque tries) est defini par limite : si celle-ci n'est pas indiquee ou plus grande que le nombre total de sources, on lui attribue ce nombre (moins 1 pour le parcours des elements)
	$this->{limite} = (!defined $this->{limite} || $this->{limite} > @{$this->{source}} ? @{$this->{source}} : $this->{limite}) - 1; 

	# Si jamais lire() a deja ete appellee
	undef $this->{labels};
	undef $this->{totaux};
	undef $this->{pourcentage};

	# Tri des sources par total
	@{$this->{source}} = sort { $b->{total} <=> $a->{total} } @{$this->{source}};

	# Construction du tableau de labels et de la hash map de valeurs
	# Pour chaque PSM::RRD
	for(0..@{$this->{source}}-1) {

		# Si nous sommes encore dans les n premiers PSM::RRD, on ajoute les valeurs
		if($_ <= $this->{limite}) {
			push(@{$this->{labels}}, @{$this->{source}}[$_]->{label});
			$this->{totaux}->{@{$this->{source}}[$_]->{label}} = @{$this->{source}}[$_]->{total};
		}

		# Le total du Groupe est calcule, lui, avec toutes les valeurs
		$this->{total} += @{$this->{source}}[$_]->{total};
	}

	# Une fois que le total du Groupe a ete calcule, on peut calculer le pourcentage que represente chaque PSM::RRD pris en compte plus haut par rapport a l'ensemble des PSM::RRD du Groupe.
	# Ce pourcentage est arrondi a l'unite.
	$this->{pourcentage}->{@{$this->{source}}[$_]->{label}} = sprintf('%.2f', @{$this->{source}}[$_]->{total} * 100 / $this->{total}) for(0..$this->{limite});
}

# Exporte les donnees sous forme d'un tableau LaTeX, en utilisant le tableau des labels et les hash map definis avec la fonction lire.
# On peut egalement demander l'affichage du graphique ou camembert correspondant (ex: tableau, tab, graphique, graph, graphTab, camembert, cam, camTab, etc)
# @return : Code LaTeX
sub toLaTeX {
	my ($this, $elements) = @_;
	my $retour;

	# On souhaite un graphique ou un camembert, on delegue la tache
	if($elements =~ /(graph|cam)/i) {
		$retour .= "\\begin{center}\n"
			."\\vspace{2mm}\n"
			.$this->{graphique}->toLaTeX($elements)
			."\\vspace{4mm}\n"
			."\\end{center}\n";
	}

	# On souhaite un tableau
	if($elements =~ /tab/i) {
		my $couleurTab;

		$retour .= "\\begin{center}\n"
			."\\vspace{2mm}\n"
			."\\rowcolors{1}{tab0}{tab1}\n"
			. "\\begin{tabular}{c|c|>{\\centering}m{6cm}|}\n";

		if($this->{label} =~ /^Niveau [0-9]+/) {
			my $categorie = $this->{label};
			$categorie =~ s/^Niveau [0-9]+//;

			$couleurTab = $this->{label};
			$couleurTab =~ s/($categorie| )//g;
			$couleurTab = 'cat'.$couleurTab;

			$this->{label} = substr($categorie, 1) if $categorie ne '';
		} else {
			$couleurTab = 'tabLabel';
		}

		$retour .= "\\hhline{~~-}\n"
		."\\rowcolor{$couleurTab}\n \\multicolumn{2}{c|}{} & \\label{$this->{label}} \\tabularnewline\n"
		."\\hhline{~-~}\n";

		# Pour chacun des labels du tableau d'instance
		for(0..@{$this->{labels}}-1) {
			my $label = @{$this->{labels}}[$_];

			# Affichage du libelle du type des donnees, eventuellement defini initialement dans le label du Groupe avec le double pipe
			# Ce libelle apparaitra a la verticale du tableau.
			$retour .= "\\cellcolor{white}\n";
			$retour .= "\\multirow{-".@{$this->{labels}}."}*{\\begin{sideways}\\indication{$this->{donnee}}\\end{sideways}}" if $_ == @{$this->{labels}}-1;

			# Affichage du label du PSM::RRD avec sa valeur en octets
			$retour .= " & \\cellcolor{$couleurTab}\n \\label".($this->{label} =~ /^(Envoy |Reu de)/ ? 'IP' : '')."{$label} & ".RRD::adapterUnite($this->{totaux}->{$label})." \\tabularnewline\n";
		}

		$retour .= "\\hhline{~--}\n"
			."\\end{tabular}\n"
			."\\vspace{6mm}\n"
			."\\end{center}\n";
	}

	return $retour;
}

# Retourne la liste des sources sous forme de string en guise d'affichage de l'instance
sub toString {
	my $this = shift;
	my $retour;

	$retour .= ' | '.$_->toString() foreach (@{$this->{source}});
	return '('.substr($retour, 3).')';
}

# Methode invoquee par perl lorsque la derniere reference de l'instance disparait
sub DESTROY {
	my $this = shift;

	undef $this->{labels};
	undef $this->{totaux};
	undef $this->{pourcentage};
	undef $_ foreach (@{$this->{source}});
	undef $this->{source};
	undef $this->{graphique};
}

1; # Obligatoire
