# Copyright (C) 2010 Julien VAUBOURG
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

use strict;
use POSIX;
use PSM::Graphique;
package RRD;


# class RRD
#
# UTILISATION
#
# La classe RRD est utilisee a chaque fois qu'un fichier de type RRD (round robin data) genere par la classe PSM::Capture intervient.
# Ses instances permettent d'avoir directement acces au nom du fichier, au label (typiquement le titre du graphique qui sera associe)
# au total des donnees qu'il a enmagasine (lu directement dans le nom du fichier : nom.total.rrd) et a l'instance du graphique
# qui lui a ete associe.


###
### VARIABLES DE CLASSE
###

our $root = "/home/sonde"; # Dossier racine du systeme de la sonde
our $debug = 0; # Mode debug : affiche les warnings (les warnings ne sont pas a eviter, ils peuvent etre prevus dans le fonctionnement du script)
our $cache = 1; # Cache active : bufferise les references des RRD qui ont deja ete instancies et des GroupeRRD correspondant aux requetes demandees via la fonction requete de PSM::Zones
our $debut; # Date de debut d'observation des RRD (timestamp unix)
our $fin; # Fin d'observation

# $resolution determine le nombre de valeurs que l'on va avoir dans les fichiers RRD
# Avec une valeur toutes les 60 secondes, defini par $nbSecondes (cf capture.pm)
# Pour 8 jours de capture, il faut prevoir (8*24*60 = 11520 minutes) soit 11520 valeurs.
our $resolution = 11520;

our @fichiersRRDSrcDst; # Mise en cache des noms de tous les fichiers RRD (ipSrc/ipDst/fichier.rrd)
our @fichiersRRDDstSrc; # Mise en cache des noms de tous les fichiers RRD (ipDst/ipSrc/fichier.rrd)

# Mise en memoire de l'arborescence du dossier des RRD, construit par PSM::Capture (le premier niveau des dossiers ont pour nom une IP source et contiennent
# chacun des dossiers ayant pour nom des IP de destination qui contiennent eux-meme differents RRD qui correspondent aux valeurs mesurees entre la source
# et la destination du contexte selon le protocole ou le port identifie). La structure de cette hash map est definie au niveau de arborescence().
our %arborescence;


###
### CONSTRUCTEUR
###

# @arg : source - Fichier RRD du dossier de RRDs genere par PSM::Capture, a partir de RRD::root/rrd (avec ou sans son total incorpore, ex : source/destination/PORT80.rrd ou source/destination/PORT80.2867864.rrd)
# @arg : label - Libelle de la donnee concernee par le RRD (sert principalement pour nommer le graphique associe)
sub new {
	my ($classe, $source, $label) = @_;

	# VARIABLES D'INSTANCE
	my $this = {
		'source' => $source,
		'label' => $label
	};

	# Le nom donne en parametre ne comprend pas le total qui le compose : on cherche donc le vrai nom de fichier qui correspond
	if($this->{source} !~ /.*\/.*\/.*\.[0-9]+\.rrd/) {
		my $regexp = '^'.substr($this->{source}, 0, rindex($this->{source}, '.')+1);
		$regexp =~ s/\./\\./g;
		my @fichiers = grep(/$regexp/, @RRD::fichiersRRDSrcDst);

		# Changement de nom de fichier
		$this->{source} = @fichiers[0] if @fichiers == 1;
		$this->{source} = '' if @fichiers == 0 || @fichiers > 1;
		print "ERREUR: Plus d'un fichier existant pour $this->{source}\n" if @fichiers > 1;

		# Si le fichier indique n'existe pas, on avorte l'instanciation en renvoyant 0 (qui permettra d'eliminer cette instance des tableaux
		# de PSM::RRD dans PSM::GroupeRRD notamment)
		if($this->{source} eq '') {
			print "ATTENTION: Aucune correspondance pour $RRD::root/rrd/$source\n" if $RRD::debug;
			return 0;
		}
	}

	# Instanciation
	bless($this, $classe);

	# Association d'un graphique
	$this->{graphique} = new Graphique($this);

	# Construction des donnees
	$this->lire();

	# Retour de la reference
	return $this;
}


###
### METHODES DE CLASSE
###

# Cette fonction met en memoire l'arborescence des RRD du dossier RRD::root/rrd, generee par PSM::Capture,
# pour un acces plus rapide ulterieurement :
#
# %RRD::arborescence = (
# 	'source0' => [destination00, destination01, destination02],
# 	'source1' => [destination10, destination11, destination12]
# );
#
# Elle cree egalement les tableaux @fichiersRRDSrcDst et @fichiersRRDDstSrc.
#
# @fichiersRRDSrcDst = (
# 	'ipSrc/ipDst/fichier.rrd',
# 	'ipSrc/ipDst/fichier.rrd',
# 	'ipSrc/ipDst/fichier.rrd'
# );
#
# @return: Reference de %RRD::arborescence
sub arborescence {

	# Si la fonction n'a jamais ete appellee, elle definie la hash map, sinon elle la renvoit directement
	if(!defined %RRD::arborescence) {
		opendir(RRDIR, "$RRD::root/rrd") or die "ERREUR: Impossible d'ouvrir $RRD::root/rrd\n";

		# Premier niveau de l'arborescence de dossiers generee par PSM::Capture : leur nom porte le nom des IP sources.
		# On parcours une  une ces IP sources en lisant le contenu dossier RRD::root/rrd.
		foreach my $dossier (readdir RRDIR) {

			# Si le fichier rencontre n'est ni un fichier cache ni un fichier .list, alors c'est un dossier IP source
			if($dossier !~ /^\./ && $dossier !~ /\.list$/) {
				opendir(ZONEDIR, "$RRD::root/rrd/$dossier") or die "ERREUR: Impossible d'ouvrir $RRD::root/rrd/$dossier\n";

				# Pour chacun des dossiers IP source on scanne son contenu pour les dossiers qui portent le nom des IP destination
				foreach my $sousDossier (readdir ZONEDIR) {

					# Le fichier rencontre n'est pas un fichier cache : c'est un dossier IP destination
					if($sousDossier !~ /^\./) {

						# On peut enfin ajouter l'IP destination au tableau de son dossier IP source ('source' => [destinations])
						push(@{%RRD::arborescence->{$dossier}}, $sousDossier);

						# On en profite pour etablir une liste de tous les fichiers RRD existants
						opendir(DESTDIR, "$RRD::root/rrd/$dossier/$sousDossier") or die "ERREUR: Impossible d'ouvrir $RRD::root/rrd/$dossier/$sousDossier\n";
						foreach (readdir DESTDIR) {
							push(@RRD::fichiersRRDSrcDst, "$dossier/$sousDossier/$_") if $_ =~ /\.rrd$/;
							push(@RRD::fichiersRRDDstSrc, "$sousDossier/$dossier/$_") if $_ =~ /\.rrd$/;
						}
						close DESTDIR;
					}
				}

				close ZONEDIR;
			}
		}

		close RRDIR;

		# Tri des noms de fichier pour bien rassembler par source ou destination
		@RRD::fichiersRRDSrcDst = sort @RRD::fichiersRRDSrcDst;
		@RRD::fichiersRRDDstSrc = sort @RRD::fichiersRRDDstSrc;
	}

	# On retourne la reference du hash map de l'arboresence
	return \%RRD::arborescence;
}

# Retourne un timestamp unix  partir d'une date "humaine"
# @arg : Date (<jj>/<mm>/<aa> <hh>h)
# @return : Timestamp
sub unixTimestamp {
	my $date = shift;
	my ($jour, $mois, $annee) = $date =~ /^([0-9]{2})\/([0-9]{2})\/([0-9]{2})$/;

	# Si la regexp n'a pas marchee c'est probablement que la date etait deja en timestamp
	return $annee ne '' ? POSIX::mktime(0, 0, 0, $jour, $mois-1, $annee+100) : $date;
}

# Retourne la date sous la forme jj/mm/aa a partir d'un timestamp unix
# @arg : Timestamp
# @return : Date formatee
sub jour {
	return POSIX::strftime("%d/%m/%y", POSIX::localtime(shift));
}

# Initialise RRD::debut et RRD::fin qui serviront tout au long du script a connaitre la periode d'observation des RRD.
# Les dates sont affinees pour correspondre a la resolution des RRD (pour qu'on lise a partir d'une valeur qui existe jusqu'a une autre valeur qui existe).
# @arg : debut - Date (<jj>/<mm>/<aaaa> <hh>h)
# @arg : fin - Idem
sub setPeriode {
	my ($debut, $fin) = @_;

	$RRD::debut = int(unixTimestamp($debut) / $RRD::resolution) * $RRD::resolution;
	$RRD::fin = int(unixTimestamp($fin) / $RRD::resolution) * $RRD::resolution;
}

# Transforme un nombre de bits en son equivalent dans l'unite qui lui correspond le mieux, en octets (ex: 8192 bits = 1024*8 bits = 1Mo)
# @arg : nombre - Nombre de bits
# @return : Nombre dans l'unite appropriee, en octets
sub adapterUnite {
	my $nombre = (shift) / 8;

	# Plusieurs gigas
	return sprintf('%.2f', $nombre / 1024**3).' Go' if $nombre > 1024**3;

	# Plusieurs megas
	return sprintf('%.2f', $nombre / 1024**2).' Mo' if $nombre > 1024**2;

	# Plusieurs kilos
	return sprintf('%.2f', $nombre / 1024).' Ko' if $nombre > 1024;

	# Plusieurs octets
	return ($nombre != 0) ? "$nombre octets" : 0;
}


###
### METHODES D'INSTANCE
###

# Lecture du RRD : simple recuperation du total cumule lisible dans nom de fichier (nom.total.rrd).
# Cette valeur est stockee dans une variable d'instance.
sub lire {
	my $this = shift;
	($this->{total}) = $this->{source} =~ /\.([0-9]+)\.rrd$/;
}

# Retourne le nom du fichier source en guise d'affichage de l'instance
sub toString {
	return shift->{source};
}

# Methode invoquee lorsque l'instance est detruite
sub DESTROY {
	undef shift->{graphique};
}

1; # Obligatoire
