# Copyright (C) 2010 Julien VAUBOURG
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

use strict;
use PSM::GroupeRRD;
package Zones;


# class Zones
#
# UTILISATION
#
# PSM::Zones represente le coeur du systeme de generation du rapport. Cette classe a pour charge de recuperer la definition des zones et des GZones contenues
# dans des fichier .hash, generes par l'interface web.
#
# Une Zone : C'est un ensemble d'IP et de sous-reseaux (definis avec leurs masques, en octal : 255.255.255.0) qui seront traitees sous un seul nom.
# 	     Chaque zone a une reference unique (entier).
# 	     Une IP peut etre dans plusieurs zones (ex: 192.168.107.2 dans une zone et 192.168.107.0/255.255.255.0 dans une autre).
#
# Les Zones sont definies dans une hash map :
#	%Zones::zones = (
#		'refZoneInt0' => ['Nom0', ['ip01', 'ip02', 'subnet01/netmask01', 'ip03']],
#		'refZoneInt1' => ['Nom1', ['subnet11/netmask11', 'ip11']]
#	);
#
# Une GZone : C'est un groupe de references de zones qui seront traitees sous un seul nom.
# 	      Chaque GZone a une reference unique (entier).
#
# Les GZones sont definies dans une hash map egalement :
#	%Zones::GZones = (
#		'refGZoneInt0' => ['GNom0', [refZoneInt0, refZoneInt1, refZoneInt2]],
#		'refGZoneInt1' => ['GNom1', [refZoneInt0, refZoneInt1]]
#	);
#
# La GZone reseau est cree dynamiquement et contient les references de toutes les zones.
# Cette GZone represente tout ce qui n'est pas internet. Tout ce qui n'est pas dans reseau est donc internet.
#
# PSM::Zones comprend une collection de methodes statiques qui permettent de manipuler ces zones.
#
# La methode requete est une methode fondamentale qui permet de renvoyer un tableau d'instances PSM::RRD des fichiers RRD qui correspondent aux criteres de la requete.
# Cette methode est utilisee pour la formation de la source de quasiment tous les PSM::GroupeRRD.
#
# Afin de parametrer ce qui est a observer dans la matrice, on defini un tableau dans un fichier de config :
# 	@Zones::matrice = (
# 		'idZone1',
# 		'idZone2',
# 		'GZONE:idGZone1'
# 	);


###
### VARIABLES DE CLASSE
###

our %zones; # Hash map des zones (explications plus haut)
our %GZones; # Hash map des GZones (explications plus haut)
our %zonesIP; # Les IP trouvees pour chaque zone sont referencees dans ce hash map : zonesIP = ('refZoneInt0' => ['ip01', 'ip02'], etc). Cette variable est initialisee par la fonction zonesIP()
our %groupes; # Le groupes sont des criteres speciaux pour les requetes : voir la fonction groupement() pour plus d'informations.
our @nomGroupes = ('PORT', 'PROTO_'); # Liste des groupes

# Appel de la fonction lire pour mettre directement les fichiers de definition des zones et GZones en memoire
lire();


###
### METHODES DE CLASSE
###

# Mise en memoire de la definition des zones et GZones a partir du fichier zones.hash, dans Zones::zones et Zones::GZones.
# Voir en haut pour la structure des hash map
sub lire {
	my $lignes;
	
	# Ouverture et parcours du zones.hash : interpretation de la definition de %Zones::zones
	open(ZONES, "$RRD::root/config/zones.hash") or die "ERREUR: Impossible d'ouvrir $RRD::root/config/zones.hash";
	$lignes .= $_ while <ZONES>;
	eval $lignes;
	close ZONES;

	$lignes = '';

	# Ouverture et parcours du zones.hash : interpretation de la definition de %Zones::GZones
	open(GZONES, "$RRD::root/config/gzones.hash") or die "ERREUR: Impossible d'ouvrir $RRD::root/config/gzones.hash";
	$lignes .= $_ while <GZONES>;
	eval $lignes;
	close GZONES;

	# Creation de la GZone reseau qui contient les references vers toutes les zones
	my @reseau = keys %Zones::zones;
	%Zones::GZones->{reseau} = ['Reseau', \@reseau];

	# Creation d'une zone qui nous permettra de recuperer les IP purement locales
	%Zones::zones->{local} = ['local', ['192.168.0.0/255.255.0.0', '172.16.0.0/255.240.0.0', '10.0.0.0/255.0.0.0']];

	# Ouverture et parcours du .tab qui servira a definir ce qui est a observer dans la matrice
	open(MATRICE, "$RRD::root/config/matrice.tab") or die "ERREUR: Impossible d'ouvrir $RRD::root/config/matrice.tab";
	$lignes = '';
	$lignes .= $_ while <MATRICE>;
	eval $lignes;
	close MATRICE;
}

# Renvoi le nom des zones auxquelles appartient une IP donnee
# @arg : ip - IP a rechercher
# @return : Reference d'un tableau de references de zones
sub quellesZones {
	my $ip = shift;
	my @retour;

	# Si la liste des IP rencontrees pour chaque zone n'est pas encore etablie (ce qui est le cas quand on est en train de l'etablir)
	if(!defined %Zones::zonesIP) {

		# Semble obligatoire pour repositionner le curseur de la hash map
		keys %Zones::zones;

		# Recherche de l'IP dans la definition de chaque zone
		while(my ($zone, $def) = each %Zones::zones) {

			# Si l'IP est dans un des sous-reseaux de la zone, on ajoute l'IP au tableau de retour (ici on considere
			# que tout ce qui est dans la definition de la zone, ce sont des sous-reseaux : estDansSousReseau se contentera de
			# comparer les IP si ca n'est pas le cas)
			push(@retour, $zone) if grep(estDansSousReseau($_, $ip), @{@$def[1]});
		}

	# Si on a la liste des IP rencontrees pour chaque zone (permet d'eviter l'appel incessant de estDansSousReseau)
	} else {
		my %ZonesIP = %Zones::zonesIP;

		# Reset du curseur	
		keys %ZonesIP;

		# Recherche de l'IP dans la liste des IP trouvees pour chaque zone
		while(my ($zone, $adresses) = each %ZonesIP) {

			# Ajout de la zone dans le tableau de retour si l'IP en fait partie
			push(@retour, $zone) if grep($_ eq $ip, @$adresses);
		}
	}

	# Retour de la reference du tableau de la retour si celui-ci n'est pas vide, sinon on considere que si l'IP n'a ete trouvee dans
	# aucune zone, c'est que c'est la "zone" internet
	return @retour != 0 ? \@retour : ['internet'];
}

# Une IP fait-elle partie d'un sous-reseau donne ?
# Cette fonction est a adapter pour les IPv6 !
# @arg : subnet - Sous-reseau avec son masque en octal (ex: 192.168.107.0/255.255.255.0),
# 	 la meme chose avec un intervale de sous-reseau (ex: 192.168.0-9.0/255.255.255.0),
# 	 juste une IP definie avec un intervale (ex: 192.168.1.2-20)
# 	 ou encore simplement une IP
# @arg : ip - IP a comparer
# @return : Booleen
sub estDansSousReseau {
	my ($subnet, $ip) = @_;

	# Le subnet est en realite une IP (ou un sous-reseau) definie avec un intervale (ex: 192.168.101-109.1)
	# On va donc chercher a comparer l'IP/le subnnet a comparer avec chacune des IP possible de l'IP/du subnet avec l'intervale
	# donnee comme subnet
	if(index($subnet, '-') != -1) {

		# Extraction de l'intervale
		my ($inter1, $inter2) = $subnet =~ /([0-9]+)-([0-9]+)/;
		$subnet =~ s/[0-9]+-[0-9]+/#/;

		# Pour chaque bit de l'intervale
		for($inter1..$inter2) {

			# Remplacement de l'intervale par le bit en cours
			my $tmp = $subnet;
			$tmp =~ s/#/$_/;

			# On s'arrete la et on retourne vrai si IP correspond a l'IP/au subnet en cours
			return 1 if estDansSousReseau($tmp, $ip);
		}

		# Si aucune IP de l'intervale n'a correspondu a l'IP a comparer, on avorte en faux
		return 0;
	}

	# Le subnet est-il bien un sous-reseau ?
	# Sinon c'est une adresse IP, une simple comparaison entre les deux suffit alors.
	if(index($subnet, '/') != -1) {
		my ($subnet, $netmask) = split(/\//, $subnet);

		# Separation des octets
		my @ip = split(/\./, $ip);
		my @subnet = split(/\./, $subnet);
		my @netmask = split(/\./, $netmask);

		# Pour chaque octet
		for(0..3) {
			# Un ET logique est effectue entre l'octet en cours de l'IP et celui du masque de sous-reseau
			my $AND = sprintf('%08b', @ip[$_]) & sprintf('%08b', @netmask[$_]);

			# Si l'octet resultant du ET logique est different de l'octet correspondant du subnet, c'est que l'IP ne peut pas appartenir a ce sous-reseau.
			# On avorte alors la fonction negativement, dans ce cas.
			return 0 if $AND != sprintf('%08b', @subnet[$_]);
		}

		# Si on est arrive jusque la sans que la fonction n'avorte, c'est que l'IP correspond au sous-reseau
		return 1;
	}

	# Le subnet indique etait une IP (probablement) : on effectue une simple comparaison
	return $subnet eq $ip;
}

# L'IP fait-elle partie de la zone donnee ?
# @arg : zone - Reference de la zone a comparer
# @arg : ip - IP a comparer
# @return : Booleen
sub estDansZone {
	my ($zone, $ip) = @_;

	# La zone est en fait l'IP donc ok
	return 1 if $ip eq $zone;

	# Ok si la zone correspond a la zone de l'IP
	return grep($_ eq $zone, @{quellesZones($ip)});
}

# L'IP fait-elle partie d'une des zones de la GZone donnee ?
# @arg : GZone - Reference de la GZone a comparer
# @arg : ip - IP a comparer
# @return : Booleen
sub estDansGZone {
	my ($GZone, $ip) = @_;

	# Si la GZone a bien une definition
	if(defined %Zones::GZones->{$GZone}) {

		# Pour chaque zone de la definition de la GZone a comparer
		foreach (@{@{%Zones::GZones->{$GZone}}[1]}) {

			# On avorte la fonction en retournant vrai si l'IP est dans une des zones
			return 1 if estDansZone($_, $ip);
		}

		# Si on est arrive jusque la, c'est que l'IP ne fait partie d'aucune des zones de la GZone
		return 0;
	}

	# La GZone n'a pas de definition correspondante (c'est une erreur et non un warning, ca ne devrait pas arriver)
	print("ERREUR: La GZone <$GZone> n'existe pas !\n");
	return 0;
}

# Trie toutes les IP rencontrees dans l'arborescence du dossier RRD::root/rrd (construite dans une hash map, par RRD::arborescence) par zones,
# dans Zones::zonesIP (voir au niveau des variables de classe pour la structure de la hash map).
# Cette hash map permet par la suite d'associer plus rapidement une IP a une zone (utile pour quellesZones()) en evitant de comparer les sous-reseaux.
# @return : Reference de %Zones::zonesIP
sub zonesIP {

	# Si la hash map n'a pas deja ete construite (sinon on renvoit directement la reference)
	if(!defined %Zones::zonesIP) {
		my %ZonesIP;

		# Premier niveau de l'arborescence : les IP sources.
		# Pour chaque IP source
		while(my ($source, $destinations) = each %{RRD::arborescence()}) {

			# Recuperation du nom des zones de cette IP
			my @zonesSrc = @{quellesZones($source)};

			# Pour chaque zone correspondant a cette IP
			foreach my $zone (@zonesSrc) {

				# Ajout de l'IP dans le tableau d'IP de l'entree du hash correspondant a la zone ('zone' => ['ip0', 'ip1', 'ip2']) si
				# elle n'a pas deja ete rencontree
				push(@{%ZonesIP->{$zone}}, $source) if !grep($_ eq $source, @{%ZonesIP->{$zone}});
			}

			# Pour chaque IP destination de cette IP source (meme processus)
			foreach my $ip (@$destinations) {

				# Recuperation du nom des zones de cette IP
				my @zonesDst = @{quellesZones($ip)};

				# Pour chaque zone correspondant a cette IP
				foreach my $zone (@zonesDst) {

					# Ajout de l'IP dans le tableau d'IP de l'entree du hash correspondant a la zone si elle n'a pas deja ete rencontree
					push(@{%ZonesIP->{$zone}}, $ip) if !grep($_ eq $ip, @{%ZonesIP->{$zone}});
				}
			}
		}

		# Definition de Zones::zonesIP a la fin pour ne pas troubler quellesZones
		%Zones::zonesIP = %ZonesIP;
	}

	# Retour de la reference de la hash map
	return \%Zones::zonesIP;
}

# Utilise zonesIP pour lister les IP trouvees pour une zone donnee
# @arg : zone - Zone a detailler
# @return : Reference vers un tableau d'IP
sub listerIP {
	my $zone = shift;
	my %ZonesIP = %{zonesIP()};

	# Renvoi du tableau d'IP qui correspond
	return %ZonesIP->{$zone} if defined %ZonesIP->{$zone};

	# La zone n'existe pas : zone est probablement une IP
	return $zone;
}

# Utilise listerIP() pour lister toutes les IP trouvees pour une GZone donnee
# @arg : GZone - GZone a detailler
# @return : Reference vers un tableau d'IP
sub listerGIP {
	my $GZone = shift;
	my @retour;

	# Si la GZone possede une definition
	if(defined %Zones::GZones->{$GZone}) {

		# Pour chaque zone de la GZone
		foreach my $zone (@{@{%Zones::GZones->{$GZone}}[1]}) {

			# Pour chaque IP de la zone
			foreach my $ip (@{listerIP($zone)}) {

				# Ajout de l'IP dans le tableau de retour si elle n'a pas deja ete ajoute (pas de doublon)
				push(@retour, $ip) if !grep($_ eq $ip, @retour);
			}
		}

	# La GZone n'existe pas : c'est une erreur, ca ne devrait pas arriver
	} else {
	        print("ERREUR: La GZone <$GZone> n'existe pas !\n");
	}

	# Retour de la reference vers le tableau d'IP
	return \@retour;
}

# Met en memoire les listes de groupes dans la variable Zones::groupes
#
# Un groupe est un ensemble de possibilites pour un type donne (ex: UDP et TCP pour le groupe PROTO_).
# Il doit avoir un fichier .list qui correspond a son nom (ex: PROTO_.list) avec simplement une possibilite par ligne.
# Il doit avoir des fichiers RRD qui sont prefixes par le nom du groupe et qui ont pour nom une des possibilites du groupe (ex: PROTO_UDP.rrd).
# Les groupes sont references dans @Zones::nomGroupes.
# Les groupes permettent simplement de savoir a l'avance toutes les possibilites qui peuvent etre rencontrees pour un type de donnees, afin de pouvoir
# correctement les rassembler. Ils permettent aussi d'en ignorer si on en supprime de la liste via l'interface web.
#
# %Zones::groupes = (
# 	'groupe0' => ['possibilite01', 'possibilite02, 'possibilite03'],
# 	'groupe1' => ['possibilite11', 'possibilite12, 'possibilite22']
# );
#
# @return : Reference vers %Zones::groupes (definition des groupes)
sub groupement {

	# Si la hash map n'a pas encore ete definie
	if(!defined %Zones::groupes) {

		# Pour chacun des groupes references dans @Zones::nomGroupes
		foreach my $groupe (@Zones::nomGroupes) {
			open(GROUPE, "$RRD::root/rrd/$groupe.list") or die "ERREUR: Impossible d'ouvrir $RRD::root/rrd/$groupe.list\n";

			# Mise en memoire des differentes valeurs trouvees
			while(<GROUPE>) {
				chomp($_);

				# Ajout dans le tableau de l'entree de la hash map correspondante ('groupe0' => ['poss0', 'poss1', 'poss2'])
				push(@{%Zones::groupes->{$groupe}}, $_);
			}
		}
	}

	# Retour de la reference de la hash map
	return \%Zones::groupes;
}

# Renvoit les differentes possibilites d'un groupe donne, a partir de %Zones::groupes
# @arg : groupe - Nom du groupe de possibilite pour lequel renvoyer la liste des possibilites (PROTO_ ou PORT)
sub groupe {
	my $groupe = shift;

	# Necessite %Zones::groupes
	my %groupes = %{groupement()};

	# Le groupe n'existe pas : c'est une erreur
	if(!defined %groupes->{$groupe}) {
		print "ERREUR: Le groupe $groupe n'existe pas\n" if $RRD::debug;
		return 0;
	}

	# Sinon, retour du tableau de possibilites correspondant au groupe
	return %groupes->{$groupe};
}

# Renvoit un tableau des references des objets RRD/GroupeRRD qui composent l'ensemble des <rrd> de <source> vers <destination>
# Si rrd est un groupe de possibilites (defini dans @Zones::nomGroupes), les RRD seront groupes par donnee, sinon par IP source (avec ipEntrants : par IP destination)
#
# 	- Je souhaite observer le trafic d'une zone a une autre, par protocoles
# 	  Il faut donc trouver tous les fichiers RRD correspondants (ipDansSource/ipDansDestination/protocoleX.rrd) pour chaque protocole,
# 	  les fusionner avec un GroupeRRD pour avoir un GroupeRRD par protocole et ensuite afficher les GroupeRRD sans les fusionner.
#	  On aura donc : Zones::requete('A', 'B', 'PROTO_')
#
#	- Je souhaite observer le trafic UDP entre deux zones
#	  On aura donc : Zones::requete('A', 'B', 'PROTO_UDP.rrd')
#
#	- Je souhaite obtenir la liste des IP de la zone A qui transmettent le plus vers la zone B
#	  Pour observer tout le trafic, on considere qu'il suffit d'additionner tous les protocoles (UDP, TCP, etc).
#	  Il faut donc directement fusionner les protocoles entre eux pour chaque dossier correspondant (ipDansSource/ipDansDestination/) et ensuite
#	  fusionner tous ces GroupeRRD, inutile donc de differencier par protocole. Les GroupeRRD prennent l'IP source pour label.
#	  On aura donc : Zones::requete('A', 'B', 'PROTO_*.rrd')
#
#	- Je souhaite observer la meme chose que precedemment mais je veux avoir les IP de la zone B qui recoivent le plus de la part de A.
#	  Les GroupeRRD auront comme label les IP de la zone destination.
#	  On aura donc : Zones::requete('A', 'B', 'PROTO_*.rrd', 'ipEntrants')
#	  Les deux algos ci-dessous expliquent la difference.
#
#
#	  Ressource avec un .rrd, sans ipEntrants :
#
#	  pour chacun des fichiers de la liste source/destination/fichier (triee) correspondant a la ressource souhaitee
#		si l'IP source actuelle n'est pas la meme que la precedente (et pas la premiere)
#			instancier un GroupeRRD avec la liste d'instances de RRD en cours
#			lui attribuer le label de l'IP source
#			l'ajouter a la liste de retour
#			vider la liste des instances de RRD
#		fin si
#
#	  	si la source et la destination du fichier correspondent aux zones definies
#	  		instancier le fichier RRD et l'ajouter a la liste de l'IP source actuelle
#	  	fin si
#	  fin pour
#
#
#	  Avec ipEntrants d'active (on inverse toutes les sources et destinations) :
#
#	  pour chacun des fichiers de la liste destination/source/fichier (triee) correspondant a la ressource souhaitee
#		si l'IP destination actuelle n'est pas la meme que la precedente (et pas la premiere)
#			instancier un GroupeRRD avec la liste d'instances de RRD en cours
#			lui attribuer le label de l'IP destination
#			l'ajouter a la liste de retour
#			vider la liste des instances de RRD
#		fin si
#
#	  	si la source et la destination du fichier correspondent aux zones definies
#	  		instancier le fichier RRD et l'ajouter a la liste de l'IP destination actuelle
#	  	fin si
#	  fin pour
#
#	  Cette fonctionnalite est principalement utilisee par la classe Flux.
#
# @arg : source - Zone, GZone, ou adresse IP
# @arg : destination - Idem
# @arg : rrd - Type de donnee souhaitee (fichier*.rrd ou groupe de possibilites)
# @arg : ipEntrants - Si vrai, les RRD seront regroupes par IP de destination plutot que par IP source
sub requete {
	my ($source, $destination, $rrd, $ipEntrants) = @_;
	my @retour;

	# Necessite l'arborescence des IP rencontrees
	my %arborescence = %{RRD::arborescence()};
	keys %arborescence;

	# Identification des eventuelles GZones
	my $GSource = $source =~ /^GZONE:/;
	my $GDestination = $destination =~ /^GZONE:/;

	# Eventuelles suppressions des marqueurs de GZone
	$source = substr($source, 6) if $GSource;
	$destination = substr($destination, 6) if $GDestination;

	# Le rrd est un groupe de possibilites (definit dans @Zones::nomGroupes)
	if($rrd !~ /\.rrd$/) {

		# Pour chaque possibilite du groupe
		foreach my $donnee (@{groupe($rrd)}) {
			my @groupe;

			# Pour chaque fichier existant qui correspond au RRD qu'on recherche
			foreach my $fichier (grep(/\/$rrd$donnee\..*\.rrd$/, @RRD::fichiersRRDSrcDst)) {

				# Recuperation de l'IP source et destination du fichier en cours
				my ($ipSrc, $ipDst) = $fichier =~ /^(.*)\/(.*)\/.*\.rrd$/;

				# Si les IP source et destination correspondent aux zones source et destination souhaites, on ajoute
				if(((!$GSource && estDansZone($source, $ipSrc)) || ($GSource && estDansGZone($source, $ipSrc))) &&
				   ((!$GDestination && estDansZone($destination, $ipDst)) || ($GDestination && estDansGZone($destination, $ipDst)))) {
					push(@groupe, new RRD($fichier, $donnee));
				}
			}

			# On ajoute la PSM::GroupeRRD compose de tous les PSM::RRD concernant la donnee dans le regroupement final
			push(@retour, new GroupeRRD(\@groupe, $donnee, 1)) if @groupe != 0;
		}

	# Le rrd est definit avec un nom de fichier RRD (eventuellement avec joker) et on ne souhaite par inverser le sens d'observation des IP
	} else {

		# Rassemble tous les RRD qui concernent l'IP en cours (source ou destination, selon ipEntrants)
		my @RRDipEnCours;

		# IP en cours (permet de voir le passage d'une IP à l'autre)
		my $ipEnCours;

		# Nom pour la recherche : on retire le .rrd pour pouvoir ajouter la partie variable (total du RRD)
		my $rrdRecherche = substr($rrd, 0, rindex($rrd, '.'));

		# Si c'est une recherche avec un joker, on adapte la regexp
		$rrdRecherche =~ s/\*/.*/;

		# Tableau de fichiers RRD utilise selon les labels souhaites (si ipEntrants et active, on souhaite donc trier par IP
		# destination)
		my @tabRecherche = $ipEntrants ? @RRD::fichiersRRDDstSrc : @RRD::fichiersRRDSrcDst;

		# Pour chaque fichier existant qui correspond au RRD qu'on recherche
		foreach my $fichier (grep(/\/$rrdRecherche\..*\.rrd$/, @tabRecherche)) {
			my $ipSrc;
			my $ipDst;
			my $rrd;

			# Recuperation de l'IP source et destination du fichier en cours
			# Si l'option ipEntrants est activee, on utilise la liste de fichiers qui inverse la source et la destination
			if($ipEntrants) {
				($ipDst, $ipSrc, $rrd) = $fichier =~ /^(.*)\/(.*)\/(.*\.rrd)$/;
			} else {
				($ipSrc, $ipDst, $rrd) = $fichier =~ /^(.*)\/(.*)\/(.*\.rrd)$/;
			}

			# Si les IP source et destination correspondent aux zones source et destination souhaites
			if(((!$GSource && estDansZone($source, $ipSrc)) || ($GSource && estDansGZone($source, $ipSrc))) &&
			   ((!$GDestination && estDansZone($destination, $ipDst)) || ($GDestination && estDansGZone($destination, $ipDst)))) {

				# Si l'IP cours n'est pas la meme que la derniere traitee (et pas la premiere), on ajoute les RRD au retour
				if(defined $ipEnCours && (!$ipEntrants && $ipEnCours ne $ipSrc || $ipEntrants && $ipEnCours ne $ipDst)) {

					# On ajoute le PSM::GroupeRRD compose de tous les fichiers RRD qui concernent l'IP en cours
					my @ajout = (@RRDipEnCours);
					push(@retour, new GroupeRRD(\@ajout, $ipEnCours, 1));
					delete @RRDipEnCours[$_] for 0..@RRDipEnCours-1;
				}

				# IP en cours
				$ipEnCours = $ipEntrants ? $ipDst : $ipSrc;

				# Instanciation du RRD et ajout dans le tableau en cours
				push(@RRDipEnCours, new RRD("$ipSrc/$ipDst/$rrd"));
			}
		}

		# Ajout du dernier paquet de RRD forme
		push(@retour, new GroupeRRD(\@RRDipEnCours, $ipEnCours, 1)) if @RRDipEnCours != 0;
	}

	# Retour de la reference du tableau de RRD/GroupeRRD
	return \@retour;
}

# Exporte les flux entre chaque zones sous forme d'un tableau LaTeX a double entree
# @return : Code LaTeX du tableau
sub matrice {
	my @matrice;
	my $retour;

	# Recuperation des zones et GZones a afficher dans la matrice
	my @tout = @Zones::matrice;

	# Entetes du tableau
	$retour .= "\\begin{center}\n"
		."\\vspace{2mm}\n"
		."\\rowcolors{2}{tab0}{tab1}\n"
		."\\begin{tabular}{>{\\columncolor{white}} c|c ".('c ' x @tout)."|}\n"
		."\\multicolumn{2}{c}{} & \\multicolumn{".@tout."}{c}{\\indication{Vers}} \\\\\n"
		."\\hhline{~~-".('-' x (@tout-1))."}\n"
		."\\rowcolor{tabLabel}\n"
		."\\multicolumn{2}{c|}{}";

	# Ajout de la ligne des labels du haut
	$retour .= " & \\begin{sideways}\\label{".Zones::label($_)."}\\end{sideways}" foreach (@tout);

	$retour .= "\\\\\n"
		."\\hhline{~-".('~' x @tout)."}\n";

	# Pour chaque zones et GZones (sources)
	for(0..@tout-1) {
		my $A = @tout[$_];

		# Ajout du label de la (g)zone consideree ici comme source
		my $ligne .= " & \\cellcolor{tabLabel}\n \\label{".Zones::label($A)."}";

		# Pour chaque zones et GZones (destination)
		foreach my $B (@tout) {

			# On observe uniquement le trafic global entre les deux zones
			my $AB = new GroupeRRD(Zones::requete($A, $B, 'PROTO_'), 1);

			# Ajout de la valeur dans le tableau
			$ligne .= " & ".($AB ? RRD::adapterUnite($AB->{total}) : 0);
		}

		$ligne .= " \\\\\n";

		# Libelle vertical pour le sens de lecture (De... Vers...)
		$retour .= "\\cellcolor{white}";
		$retour .= "\\multirow{-".@tout."}*{\\begin{sideways}\\indication{De}\\end{sideways}} " if $_ == @tout-1;
		$retour .= $ligne;
	}

	$retour .= "\\hhline{~-".('-' x @tout)."}\n"
		."\\end{tabular}\n"
		."\\vspace{6mm}\n"
		."\\end{center}\n";

	# Retour du code LaTeX
	return $retour;
}

# Formatage des labels des tableaux pour le LaTeX (pour internet les GZones)
sub label {
	my $zone = shift;

	# Internet prend simplement une majuscule quand c'est un label
	return 'Internet' if $zone eq 'internet';

	# Une GCategorie
	return 'Niveau '.substr($zone, 5) if $zone =~ /^GCAT_/;

	# Une categorie (suppression du marqueur, plus une separation des mots en fonction des underscores)
	if($zone =~ /^CAT_/) {
		$zone = substr($zone, 4);
		my $retour;
		$retour .= ' '.ucfirst($_) foreach (split(/_/g, $zone));

		return substr($retour, 1);
	}

	# Les GZones perdent leur marqueur pour etre affichees
	return ($zone =~ /^GZONE:/) ? @{%Zones::GZones->{substr($zone, 6)}}[0] : @{%Zones::zones->{$zone}}[0];
}

1; # Obligatoire
