#!/usr/bin/perl

# Copyright (C) 2010 Julien VAUBOURG
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

use lib '/home/sonde/classes';
use strict;

use PSM::RRD;
use PSM::Zones;
use PSM::LaTeX;
use PSM::Flux;
use PSM::CategoriesInternet;


# UTILISATION
#
# Ce script lit l'arborescence des fichiers RRD pour construire un rapport detaille des flux entre les zones et GZones definies
# dans zones.hash et en fonction de la configuration du rapport demandee via l'interface web.
#
# ./rapport.pl rapport.tex : Le rapport final, genere dans rapport/, s'appellera rapport.tex et le PDF rapport.pdf.
#			    Cet argument doit finir par .tex, est facultatif (par defaut, rapport.tex) et doit toujours se trouver en premier.
# ./rapport.pl nocache : Lorsque le cache est active (par defaut), les instances de PSM::RRD sont archivees et partagees lorsqu'on souhaite en recreer
# 			 une qui a deja ete generee. Les requetes de la fonction requete() de PSM::Zones sont egalement archivees pour repondre plus
# 			 vite lorsqu'on souhaite la meme formation de RRD/GroupeRRD. Enfin, les images des graphiques et camemberts sont recuperees si elles
# 			 ont deja ete tracees. L'option nocache desactive ces trois fonctionnalites.
# ./rapport.pl debug : Le debugage est desactive par defaut. L'option debug permet d'afficher sur la sortie standard les warnings et la gestion du cache (mise en
# 		       cache ou recuperation du cache). Les warnings interviennent surtout lorsqu'on cree un PSM::RRD ou PSM::GroupeRRD qui n'a aucune source
# 		       d'existante. Les warnings sont naturels lorsqu'on fait des requetes par type, il ne faut pas chercher a les eliminer, ils permettent juste
# 		       de voir ce qui se passe.
# ./rapport.pl pdf : Le fichier .tex sera automatiquement compile en PDF dans le repertoire rapport/. Deux compilations successives sont effectuees (pour la table
#		     des matieres et le nombre de page, c'est un comportement normal de LaTeX).
# ./rapport.pl matrice : Une matrice des flux (definis dans matrice.tab) sera ajoutee a la fin du document
# ./rapport.pl noweb : Pas d'analyses des sites internet visites lorsqu'on trouve internet dans un flux
#
# Ormis le nom du fichier tex qui doit toujours etre en premier (mais qui est facultatif),
# les autres options peuvent etre definies dans n'importe quel ordre.


# Retourne le code LaTeX de toutes les informations possibles entres deux zones, GZones ou ip
# @arg : A - Zone, GZone ou IP
# @arg : B - Idem
sub flux {
	my ($A, $B) = @_;
	my $retour;

	# Recuperation des noms de zones formates pour l'affichage
	my $labelA = Zones::label($A);
	my $labelB = Zones::label($B);

	# Pour voir la bande passante entre les deux zones, on dit à chaque couple ipSrc/ipDest concernes de fusionner
	# leurs bases qui concernent des protocoles (PROTO*.rrd) et on fusionne ensuite toutes ces fusions pour ne faire
	# plus qu'une courbe (GroupeRRD avec un 1). En fusionnant tous les protocoles, on voit tout le trafic.
	my $BP_AB = new GroupeRRD(Zones::requete($A, $B, 'PROTO*.rrd'), "Trafic de $labelA vers $labelB", 1);
	my $BP_BA = new GroupeRRD(Zones::requete($B, $A, 'PROTO*.rrd'), "Trafic de $labelB vers $labelA", 1);

	# Pour afficher les flux entre les zones en distinguant les protocoles utilises, on prend les protocoles un
	# par un et on rassemble tous les RRD qui les concernent, de chaque couple ipSrc/ipDest concerne, on les fusionne
	# (PROTO_). On rassemble ensuite ces protocoles, sans les fusionner (GroupeRRD avec 0).
	# Puisqu'on veut avoir cette information dans un meme tableau pour A vers B et B vers A, on demande a Flux de
	# s'occuper de tout ca.
	my $PROTOS = new Flux($A, $B, 'PROTO_', 'Prot.', 4);
	my $PORTS = new Flux($A, $B, 'PORT', 'App.');

	# On souhaite afficher les graphiques de l'information plus haut.
	# Le systeme de cache des requetes se contentera de recuperer ces deux requetes puisqu'elles ont deja ete crees
	# par le Flux precedent.
	my $PORTS_AB = new GroupeRRD(Zones::requete($A, $B, 'PORT'), "$labelA vers $labelB (top 10)", 0, 10);
	my $PORTS_BA = new GroupeRRD(Zones::requete($B, $A, 'PORT'), "$labelB vers $labelA (top 10)", 0, 10);

	# On souhaite observer les flux entre les deux zones, par protocoles et par IP. On demande donc a Flux de nous
	# faire un recapitulatif. En indiquant un ressource avec un joker a Flux, il sait qu'on veut un detail par IP.
	my $IP_AB = new Flux($A, $B, 'PROTO_*.rrd', 'IP', 10);
	my $IP_BA = new Flux($B, $A, 'PROTO_*.rrd', 'IP', 10);

	my $aucunedonnee = "\\emph{Aucune donnée}\\\\\n";

	# Export des informations en LaTeX
	if(!$BP_AB && !$BP_BA) {
		$retour .= $aucunedonnee;
	} else {
		$retour .= "\\subsection{Bande passante utilisée}\n"
			.($BP_AB ? $BP_AB->toLaTeX('graphique') : $aucunedonnee)
			.($BP_BA ? $BP_BA->toLaTeX('graphique') : $aucunedonnee)
			."\\subsection{Répartition TCP / UDP}\n\n"
			.($PROTOS ? $PROTOS->toLaTeX('camTab') : $aucunedonnee)."\n"
			."\\subsection{Détails par port}\n"
			.($PORTS_AB ? $PORTS_AB->toLaTeX('graphique') : $aucunedonnee)
			.($PORTS_BA ? $PORTS_BA->toLaTeX('graphique') : $aucunedonnee)
			.($PORTS ? $PORTS->toLaTeX('camTab') : $aucunedonnee)
			."\\subsection{Détails par IP}\n"
			.($IP_AB ? $IP_AB->toLaTeX('tableau') : $aucunedonnee)
			.($IP_BA ? $IP_BA->toLaTeX('tableau') : $aucunedonnee);
	}

	return $retour;
}

# Analyse du trafic internet
# @arg: zone - Zone ou GZone pour laquelle le trafic internet est a observer (top serveurs visites par categories de sites critiques, etc)
# @return : Code LaTeX de l'analyse internet
sub internet {
	my $zone = shift;
	my $retour;

	# Camembert de la repartition des sites vers les niveaux de sites critiques
	my $camembertParGCategories = CategoriesInternet::toLaTeX('camembertParGCategories', $zone, 10);
	my $aucunedonnee = "\\emph{Aucune donnée}\\\\\n";

	# Si le camembert a ete cree, c'est qu'il y a d'autres donnees a afficher
	if($camembertParGCategories) {

		# Top des categories au sein des niveaux (les GCategories, qui regroupent des categories), avec un indice volumetrique
		# du download depuis internet
		my $topCategoriesParGCategorie = CategoriesInternet::toLaTeX('topCategoriesParGCategorie', $zone, 10);

		# Top des sites visites par categorie, pour toutes les categories qui en on reconnus
		my $topServeursParCategorie = CategoriesInternet::toLaTeX('topServeursParCategorie', $zone, 5);

		# Top des sites critiques visites toutes categories confondues
		my $topGeneralParServeur = CategoriesInternet::toLaTeX('topGeneralParServeur', $zone, 20);

		$retour .= "\\subsubsection{Répartition des catégories de sites critiques visités}\n"
			.$camembertParGCategories
			."\\subsubsection{Tops 10 des catégories critiques par niveau d'alerte}\n"
			.($topCategoriesParGCategorie ? $topCategoriesParGCategorie : $aucunedonnee)
			."\\subsubsection{Tops 5 des noms de domaine par catégorie}\n"
			.($topServeursParCategorie ? $topServeursParCategorie : $aucunedonnee)
			."\\subsubsection{Tops 20 des noms de domaine critiques visités}\n"
			.($topGeneralParServeur ? $topGeneralParServeur : $aucunedonnee);
	} else {
		$retour .= $aucunedonnee;
	}

	return $retour;
}

# Si l'option noweb n'est pas indiquee, on prepare les zones pour les modules web (etude du trafic internet par categories).
# Ceci peut prendre beaucoup de temps ! Mais c'est ce qui permet de realiser les modules web ensuite en quelques secondes.
if(!grep($_ =~ /noweb/i, @ARGV)) {
	print `date "+[%d/%m/%y - %H:%M:%S] : Generation des zones de categories"`;

	print "Mise en cache des noms serveurs...\n" if $RRD::debug;
	CategoriesInternet::miseEnCacheNomsServeurs();

	print "Mise en cache des categories...\n" if $RRD::debug;
	CategoriesInternet::miseEnCacheDesCategories();

	print "Generation des zones...\n" if $RRD::debug;
	CategoriesInternet::genererZonesCategories();
}

# L'option nocache du script interdit la mise en cache des instances de PSM::RRD et des requetes de PSM::Zones et la recuperation des images
$RRD::cache = !grep($_ =~ /nocache/i, @ARGV);

# L'option debug affiche les warnings et la gestion du cache
$RRD::debug = grep($_ =~ /debug/i, @ARGV);

# Si aucun nom de rapport n'a été donné en premier argument (ou qu'il ne finit pas par .tex), on nomme le rapport rapport.tex
my $nomRapport = @ARGV[0] =~ /\.tex$/ ? @ARGV[0] : 'rapport.tex';

# Creation d'un objet de rapport LaTeX
our $rapport = new LaTeX($nomRapport);

# Ecriture du preambule LaTeX dans le fichier
$rapport->ecrirePreambule();

# Recuperation de la liste des flux a observer (initialisation de @flux)
my $lignes;
my @flux;
open(FLUX, "$RRD::root/config/flux.tab") or die "ERREUR: Impossible d'ouvrir $RRD::root/config/flux.tab\n";
$lignes .= $_ while <FLUX>;
eval $lignes;
close FLUX;

# Construction du rapport en fonction des flux a observer
# Pour chaque couple de zones
FLUX foreach (@flux) {
	my ($A, $B) = @$_;

	# Recuperation des noms de zones
	my $zoneA = Zones::label($A);
	my $zoneB = Zones::label($B);

	# Avorte l'observation de ce flux si une des deux parties du flux n'existe pas
	if($zoneA eq '' || $zoneB eq '') {
		print "ATTENTION: Demande d'observation entre $A et $B, or ".($zoneA eq '' ? $A : $B)." n'existe plus\n" if $RRD::debug;
		next FLUX;
	}

	# Volonte de regarder du trafic internet ?
	my $internet = $A eq 'internet' ? $B : ($B eq 'internet' ? $A : 0);

	print `date "+[%d/%m/%y - %H:%M:%S] : Entre <$zoneA> ($A) et <$zoneB> ($B)"`;

	# Ajout au rapport
	$rapport->add("\\section{Entre $zoneA et $zoneB}\n");
	$rapport->add(flux($A, $B));

	if($internet && !grep($_ =~ /noweb/i, @ARGV)) {
		print `date "+[%d/%m/%y - %H:%M:%S] : Entre <$zoneA> ($A) et <$zoneB> ($B) - Module web"`;
		$rapport->add("\\subsection{Analyse des sites internet critiques visités}\n");
		$rapport->add(internet($internet));
	}

	$rapport->ecrire();
}

# Ajout d'une matrice des flux entre chaque zone
if(grep($_ =~ /matrice/i, @ARGV)) {
	$rapport->add("\\section{Matrice des flux}\n");
	$rapport->add(Zones::matrice());
	$rapport->ecrire();
}

# Finalisation du fichier .tex
$rapport->finaliser();
print "Fichier .tex genere\n";

# Le rapport doit etre compile (option pdf de la ligne de commande)
if(grep($_ =~ /pdf/i, @ARGV)) {
	# Generation du .pdf
	# Deux compilations pour la generation de la table des matiere et le nombre de page
	$rapport->compiler();
	$rapport->compiler();
}

print "Fichier PDF ecrit dans le dossier rapport !\n";

