# Copyright (C) 2010 Julien VAUBOURG
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

use strict;
use PSM::RRD;
use PSM::MD5;
package Graphique;


# class Graphique
#
# UTILISATION
#
# PSM::Graphique s'occupe de tracer des graphiques et des camemberts a partir d'un PSM:RRD ou d'un PSM::GroupeRRD.
# Chaque RRD et GroupeRRD possede une variable d'instance <graphique> de type PSM::Graphique.
# 
# Les graphiques :
# Ils sont generes grace a la commande graph de rrdtool, a partir des fichiers rrd directement (et non les valeurs des tableaux des instances).
# La seule tache de PSM::Graphique est de definir les options de RRDTool, courbes() s'est occupe. Si la source est un GroupeRRD et que sa variable
# fusion est a vrai, les courbes sont additionnees et seule la courbe qui les additionne toutes sera affichee. Sinon, toutes les courbes seront parametrees
# et elle seront toutes affichees (sous forme de lignes). Le format de sortie est en PDF pour obtenir une image vectorielle et qui peut s'inserer dans
# un document latex.
#
# Les camemberts :
# Pour avoir de jolis camemberts on utilise un programme java de ma conception qui generera une image au format PNG (le vectoriel n'etant pas possible...).
# On utilise ensuite convert, de la suite imagemagick, pour supprimer les blancs autour de l'image. A noter qu'un camembert n'a aucun sens avec un PSM::RRD.


###
### VARIABLES DE CLASSE
###

our $dir = '/home/sonde/graphiques'; # Dossier temporaire de stockage des images
our @couleurs = (qw(7b2fb1 c97c21 2d3ab5 ae1f0c e1d738 51e7de 17701b 223b35 685b10 13f00d 50c5cf)); # Set de couleurs pour la generations des graphiques et des camemberts
our $width = 400; # Largeur des graphiques (px)
our $height = 165; # Hauteur des graphiques (px)
our $camWidth = 250; # Largeur des camemberts (px)
our $camHeight = 250; # Hauteur des camemberts (px)
our $nbMaxDansCam = 4; # Nombre max de valeurs representees sur un camembert


###
### CONSTRUCTEUR
###

# @arg : rrd - PSM::RRD ou PSM::GroupeRRD a exploiter pour la generation du graphique
sub new {
	my ($classe, $rrd) = @_;

	# VARIABLES D'INSTANCE
	my $this = {
		'rrd' => $rrd,

		# Base utilisee pour le nom des images.
		# L'utilisation d'un MD5 permet d'associer une image a une liste precise de fichiers sur une periode periode precise.
		# Ceci permet de verifier que l'image n'existe pas deja avant de la generer.
		'baseImage' => 'ns-graph-'.MD5->hexhash($RRD::debut.$RRD::fin.$rrd->toString().$rrd->{fusion})
	};

	# Instanciation
	bless($this, $classe);

	# Retour de la refence
	return $this;
}


###
### METHODES D'INSTANCE
###

# Definit les options de RRDTool graph pour la creation d'un graphique et les stock dans la variable d'instance <options>
# @return : Contenu de <options>
sub options {
	my $this = shift;

	# Si options() n'a pas deja ete appellee
	if(!defined $this->{options}) {

		# Options de RRDTool graph
		$this->{options} = " -s $RRD::debut -e $RRD::fin"
			.' --vertical-label "bits par seconde"'
			.' --imgformat PDF'
			.' --font DEFAULT:0:Helvetica'
			.' --font TITLE:8:Helvetica-Bold'
			.($this->{rrd}->{label} ne '' ? " --title \"$this->{rrd}->{label}\"" : '')
			." -w ".($Graphique::width)." -h ".($Graphique::height)
			.' --color CANVAS#DFDFDF'
			.' --color BACK#EEEEEE'
			.' --color FONT#000000'
			.' --color MGRID#111111'
			.' --color GRID#4C4C4C'
			.' --color FRAME#CECECE'
			.' --color ARROW#000000'
			.' --color SHADEA#FFFFFF'
			.' --color SHADEB#FFFFFF';
	 }

	# Retour des options
	return $this->{options};
}

# Definit les options de RRDTool graph pour tracer les courbes du graphique (si c'est un graphique qu'on souhaite), et le stock dans la variable
# d'instance <courbes>
# @arg Op. : option - Pour une utilisation interne liee a la recursivite :  permet de ne renvoyer que la definition des courbes, ou forcer l'ajout d'un label
# @return : Contenu de <courbes>
sub courbes {
	my ($this, $option) = @_;

	# DEF dans $option permet de ne renvoyer que la definition des courbes (pas d'option d'affichage) et de forcer la fusion
	my $DEF = $option eq 'DEF';

	# LABEL dans $option permet de forcer l'ajout du label du PSM::RRD dans l'affichage, meme si le GroupeRRD demande la fusion
	my $LABEL = $option eq 'LABEL';
	my $affichage;

	# Uniquement si le boulot n'a pas encore ete fait ou si on est dans un calcul interne (appelle par une autre fonction courbes) et qu'on souhaite
	# la fusion exceptionnelle ou l'apparition exceptionnelle du label
	if(!defined $this->{courbes} || ($DEF && !defined $this->{courbesDef}) || $LABEL) {
		$this->{courbesDef} = '';

		# Le RRD est un GroupeRRD (plusieurs sources)
		if($this->{rrd}->{source} =~ /^ARRAY/) {

			# Pas de fusion des courbes
			if(!$this->{rrd}->{fusion} && !$DEF)  {

				# Pour chaque source, on ajoute simplement la definition de ses courbes, afin de toutes les superposer
				for(0..$this->{rrd}->{limite}) {

					# On recupere les courbes du graphique de la source, en forcant le LABEL, meme si les courbes ont deja ete generees
					# sans label (fusion)
					my $courbes = @{$this->{rrd}->{source}}[$_]->{graphique}->courbes('LABEL');

					# Lorsqu'il y a plusieurs courbes a afficher sur un meme graphique , on transforme les AREA en LINE1 pour n'avoir
					# que des lignes et qu'aucune donnee n'en cache une autre.
					$courbes =~ s/AREA/LINE1/g;

					# Ajout de la courbe
					$this->{courbes} .= $courbes;
				}

			# Option fusion du PSM::RRD activee, ou bien DEF est declare : on souhaite fusionner les sources
			} else {
				my $cdef;
				my $plus;

				# Pour chaque source, on additionne ses donnees afin de ne produire qu'une seule courbe a afficher
				for(0..$this->{rrd}->{limite}) {

					# On recupere la definition de LA courbe de la source (DEF fait fusionner)
					my $courbes = @{$this->{rrd}->{source}}[$_]->{graphique}->courbes('DEF');

					# On recupere son nom
					my ($courbe) = substr($courbes, rindex($courbes, 'CDEF')) =~ /^CDEF:(c[0-9]+)=/;

					# On ajoute aux courbes a additionner
					$this->{courbesDef} .= $courbes;
					$plus .= $cdef ne '' ? ',+' : '';
					$cdef .= ",$courbe";
				}

				# Affichage de la courbe qui additionnera toutes les autres (le rand assure un nom de courbe presque unique)
				my $courbe = 'c'.int(rand 1000000).int(rand 100000).int(rand 100000);
				$this->{courbesDef} .= " CDEF:$courbe=".substr($cdef, 1).$plus;
				$affichage .= " AREA:$courbe#<COULEUR>:\"".(!$this->{rrd}->{fusion} || $LABEL ? $this->{rrd}->{label} : '')."\"";

				# Affichage du maximum atteint par la courbe (si ca n'est pas un calcul interne)
				$affichage .= " CDEF:${courbe}Kbps=$courbe,1000,/ COMMENT:\"Maximum \\: \" VDEF:debitMax=${courbe}Kbps,MAXIMUM GPRINT:debitMax:\"%.2lf kbps\\l\"" if !$LABEL;
			}

		# Le RRD est un objet RRD
		} else {
			my $courbe = 'c'.int(rand 1000000).int(rand 100000).int(rand 100000);

			# Definition de la courbe, et remplacement des valeurs UNKNOWN (nan) par 0 (sinon x+nan=nan)
			$this->{courbesDef} = " DEF:${courbe}_=$RRD::root/rrd/$this->{rrd}->{source}:cpt:AVERAGE"
				." CDEF:$courbe=${courbe}_,UN,0,${courbe}_,IF";
	
			# Ajout de l'affichage si on est pas dans une recursion liee a un calcul
			$affichage = " AREA:${courbe}_#<COULEUR>:\"$this->{rrd}->{label}\"";
		}

		# Initialisation de <courbes> uniquement si elle n'existe pas deja (sinon c'est que nous sommes dans le cas d'un graphique superpose qui additionne dans le cadre d'une recursion liee a un calcul, indique par DEF)
		$this->{courbes} = $this->{courbesDef}.$affichage if !defined $this->{courbes} && !$LABEL;
	}

	# Si nous sommes dans le cas d'un calcul, on ne retourne que les definition des courbes
	return $DEF || $LABEL ? $this->{courbesDef}.($LABEL ? $affichage : '') : $this->{courbes};
}

# Cree l'image du graphique resultante en remplacant les couleurs des courbes
sub tracer {
	my ($this, $type) = @_;
	my $parametres;

	# Determination du nom de l'image
	$this->{image} = $this->{baseImage}.'Cam.pdf' if $type =~ /cam/i;
	$this->{image} = $this->{baseImage}.'.pdf' if $type !~ /cam/i;

	# Si l'image n'a pas deja ete tracee ou que le cache est active, alors on trace (sinon on recupere l'image)
	if(!-f "$Graphique::dir/$this->{image}" || !$RRD::cache) {

		# Le graphique n'est pas un camembert
		if($type !~ /cam/i) {

			# Recuperation des parametres a indiquer a RRDTool graph
			$parametres = $this->options().$this->courbes();
			my $i = 0;

			# Attribution de couleurs aux courbes
			$parametres =~ s/<COULEUR>/@Graphique::couleurs[$i++]/ while($parametres =~ /<COULEUR>/);

		# On veut un camembert
		} else {

			# On verifie que la source est bien un PSM::GroupeRRD (pour un camembert vaut mieux...)
			if($this->{rrd}->{source} =~ /^ARRAY/) {
				my @labels;
				my @pourcentages;
				my $pourcentage = 0;

				# Construction d'un tableau avec les labels du GroupeRRD et d'un autre tableau qui contiendra les totaux aux memes indices
				my $limite = $this->{rrd}->{limite} > $Graphique::nbMaxDansCam-2 ? $Graphique::nbMaxDansCam-2 : $this->{rrd}->{limite};
				for (0..$limite) {
					my $label = @{$this->{rrd}->{labels}}[$_];

					push(@labels, $label." %28$this->{rrd}->{pourcentage}->{$label}%25%29");
					push(@pourcentages, $this->{rrd}->{pourcentage}->{$label});
					$pourcentage += $this->{rrd}->{pourcentage}->{$label};
				}

				# Afin que les pourcentages des etiquettes soient proportionnels aux parts du camemberts, on ajoute une part qui represente
				# toutes les autres parts qui ne sont pas visibles
				if(100-$pourcentage > 0) {
					my $pourcentageDivers = sprintf('%.2f', 100-$pourcentage);
					push(@labels, "Divers %28$pourcentageDivers%25%29");
					push(@pourcentages, $pourcentageDivers);
				}

				# On aplati les tableaux pour pouvoir les mettre en parametre du programme java
				$parametres = '?label='.join('*', @labels).'&data='.join('*', @pourcentages).(@labels[0] =~ /Niveau [0-9]+/ ? '&categories' : '');

			# La source n'est pas un GroupeRRD (ce qui est debile...)
			} else {

				# Attribution du seul label avec son total
				$parametres = '?label='.$this->{rrd}->{label}.'&data=100';

				print "ATTENTION: Une seule valeur dans le camembert" if $RRD::debug;
			}
		}

		# Generation d'un graphique de temps avec RRDTool graph
		if($type !~ /cam/i) {
			open(CMD, ">$Graphique::dir/ns-graph-cmd") or die "ERREUR: Impossible d'ouvrir $Graphique::dir/ns-graph-cmd\n";
			print CMD "rrdtool graph $Graphique::dir/$this->{image} $parametres";
			close CMD;

			`sh $Graphique::dir/ns-graph-cmd`;

		# Generation d'un camembert avec le petit programme java
		} else {
			my $imgTmp = "$Graphique::dir/$this->{image}";
			$imgTmp =~ s/\.pdf$/.png/;

			`wget -q "http://127.0.0.1/camemberts/arti.php$parametres" -O $imgTmp`;
			`convert $imgTmp $Graphique::dir/$this->{image} && rm $imgTmp`;
		}

		print "CREATION DE: $Graphique::dir/$this->{image}\n" if $RRD::debug;
	} else {
		print "RECUPERATION DE: $Graphique::dir/$this->{image}\n" if $RRD::debug;
	}
}

# Export en LaTeX de l'image du graphique, apres l'avoir trace
sub toLaTeX {
	my ($this, $type) = @_;

	# Creation ou recuperation de l'image
	$this->tracer($type);

	# Code LateX
	return $type =~ /cam/ ?
		"\\includegraphics[width=${camWidth}pt, height=${camHeight}pt]{$Graphique::dir/$this->{image}}\n" :
		"\\includegraphics{$Graphique::dir/$this->{image}}\n";
}

# Methode invoquee lors de la destruction de l'instance
sub DESTROY {
	my $this = shift;
	undef $this->{rrd};
}

1;
